// 🎯 Dart imports:
import 'dart:typed_data';

// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:verttice_models/verttice_models.dart';

class ExpandableOptionsConfig {
  final Future<List<String>> Function({
    required String optionsID,
  }) getOptions;

  final Future<void> Function({
    required String optionsID,
    required String option,
  }) setOption;

  final Future<void> Function({
    required String optionsID,
    required String option,
  }) deleteOption;

  ExpandableOptionsConfig({
    required this.getOptions,
    required this.setOption,
    required this.deleteOption,
  });
}

class VertticeUIConfig {
  VertticeUIConfig._privateConstructor();
  static final VertticeUIConfig instance =
      VertticeUIConfig._privateConstructor();

  late ExpandableOptionsConfig _expandableOptionsConfig;

  late Future<List<FileModel>> Function({
    required BuildContext context,
    required List<UploadFileDataModel> files,
  }) _uploadFiles;

  late Future<String> Function({
    required String path,
  }) _getStorageDownloadURL;

  late Future<Uint8List?> Function({
    required String path,
  }) _getStorageData;

  late String _firebaseApiKey;

  ExpandableOptionsConfig get expandableOptionsConfig =>
      _expandableOptionsConfig;

  Future<List<FileModel>> Function({
    required BuildContext context,
    required List<UploadFileDataModel> files,
  }) get uploadFiles => _uploadFiles;

  Future<String?> Function({
    required String path,
  }) get getStorageDownloadURL => _getStorageDownloadURL;

  Future<Uint8List?> Function({
    required String path,
  }) get getStorageData => _getStorageData;

  String get firebaseApiKey => _firebaseApiKey;

  initExpandableOptionsConfig(ExpandableOptionsConfig expandableOptionsConfig) {
    _expandableOptionsConfig = expandableOptionsConfig;
  }

  initUploadFiles({
    required Future<List<FileModel>> Function({
      required BuildContext context,
      required List<UploadFileDataModel> files,
    })
        uploadFiles,
  }) {
    _uploadFiles = uploadFiles;
  }

  initGetFiles({
    required Future<String> Function({
      required String path,
    })
        getStorageDownloadURL,
    required Future<Uint8List?> Function({
      required String path,
    })
        getStorageData,
  }) {
    _getStorageDownloadURL = getStorageDownloadURL;
    _getStorageData = getStorageData;
  }

  initConfig({
    required String firebaseApiKey,
  }) {
    _firebaseApiKey = firebaseApiKey;
  }

  static const mobileWidth = 750;
  static const mobileHeight = 500;
}
