// 🎯 Dart imports:
import "dart:async";

// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:cross_connectivity/cross_connectivity.dart';
import 'package:fluent_ui/fluent_ui.dart' as fluent;
import 'package:shared_preferences/shared_preferences.dart';
import 'package:wakelock/wakelock.dart';

class VertticeUIState extends ChangeNotifier {
  VertticeUIState() {
    _initTheme();
    _initConnectivity();
    _initDeviceAwake();
  }

  // // // VARS
  bool _connectedToTheInternet = false;
  bool _keepDeviceAwake = true;
  ThemedColors _themedColors = ThemedColors(
    accentColor: fluent.Colors.red,
    textColor: Colors.white,
    accentColorName: 'red',
  );
  bool _isDarkTheme = false;

  // // //  GETS
  bool get connectedToTheInternet => _connectedToTheInternet;
  bool get keepDeviceAwake => _keepDeviceAwake;
  bool get isDarkTheme => _isDarkTheme;
  ThemedColors get themedColors => _themedColors;
  Color get brightnessColor => isDarkTheme ? Colors.grey[850]! : Colors.white;

  // // //  SETS

  Future<void> setKeepDeviceAwake(bool val) async {
    final pref = await SharedPreferences.getInstance();
    await pref.setBool("awake", val);
    Wakelock.toggle(enable: val);
    _keepDeviceAwake = val;
    notifyListeners();
  }

  void setConnectToTheInternet(bool val) {
    _connectedToTheInternet = val;
    notifyListeners();
  }

  Future _initDeviceAwake() async {
    final pref = await SharedPreferences.getInstance();
    final awake = pref.getBool("awake");
    if (awake == null) {
      await pref.setBool("awake", true);
    }
    setKeepDeviceAwake(awake ?? true);
  }

  Future<void> _initConnectivity() async {
    final connectivityResult = await (Connectivity().checkConnectivity());
    setConnectToTheInternet(connectivityResult != ConnectivityStatus.none);
    Connectivity().onConnectivityChanged.listen((ConnectivityStatus result) {
      setConnectToTheInternet(result != ConnectivityStatus.none);
    });
  }

  Future<void> setThemedColors(String color) async {
    final pref = await SharedPreferences.getInstance();
    await pref.setString("themedColors", color);
    _themedColors = _colorSwitch(color);

    notifyListeners();
  }

  Future<void> setBrightnessTheme(bool isDark) async {
    final pref = await SharedPreferences.getInstance();
    await pref.setBool("dark", isDark);
    _isDarkTheme = isDark;
    notifyListeners();
  }

  Future _initTheme() async {
    final pref = await SharedPreferences.getInstance();
    final dark = pref.getBool("dark") ?? false;
    final themedColors = pref.getString("themedColors") ?? 'red';
    await setThemedColors(themedColors);
    await setBrightnessTheme(dark);
  }

  ThemedColors _colorSwitch(String color) {
    switch (color) {
      case 'orange':
        return ThemedColors(
          accentColor: fluent.Colors.orange,
          textColor: Colors.white,
          accentColorName: 'orange',
        );

      case 'magenta':
        return ThemedColors(
          accentColor: fluent.Colors.magenta,
          textColor: Colors.black,
          accentColorName: 'magenta',
        );

      case 'purple':
        return ThemedColors(
          accentColor: fluent.Colors.purple,
          textColor: Colors.white,
          accentColorName: 'purple',
        );
      case 'blue':
        return ThemedColors(
          accentColor: fluent.Colors.blue,
          textColor: Colors.black,
          accentColorName: 'blue',
        );
      case 'teal':
        return ThemedColors(
          accentColor: fluent.Colors.teal,
          textColor: Colors.black,
          accentColorName: 'teal',
        );
      case 'green':
        return ThemedColors(
          accentColor: fluent.Colors.green,
          textColor: Colors.white,
          accentColorName: 'green',
        );

      default:
        return ThemedColors(
          accentColor: fluent.Colors.red,
          textColor: Colors.white,
          accentColorName: 'red',
        );
    }
  }
}

class ThemedColors {
  final Color textColor;
  final Color accentColor;
  final String accentColorName;

  ThemedColors({
    required this.textColor,
    required this.accentColor,
    required this.accentColorName,
  });
}
