// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class ScreenHandler extends StatelessWidget {
  final Widget? mobileWidget;
  final Widget? desktopWidget;
  final bool keepStateAlive;

  const ScreenHandler({
    Key? key,
    this.mobileWidget,
    this.desktopWidget,
    this.keepStateAlive = false,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(builder: (context, constraints) {
      final screenType = ScreenSizeUtils.screenTypeFromSize(
        constraints.maxWidth == double.infinity
            ? MediaQuery.of(context).size.width
            : constraints.maxWidth,
      );
      if (!keepStateAlive) {
        return [
          desktopWidget ??
              const Center(
                child: ThemedText(
                  "No Desktop screen for this page :(",
                  showThemed: true,
                  useScreenConstraints: false,
                ),
              ),
          mobileWidget ??
              const Center(
                child: ThemedText(
                  "No Mobile screen for this page :(",
                  showThemed: true,
                  useScreenConstraints: false,
                ),
              )
        ][screenType.index];
      }
      return IndexedStack(
        index: screenType.index,
        children: [
          desktopWidget ??
              const Center(
                child: ThemedText(
                  "No Desktop screen for this page :(",
                  showThemed: true,
                  useScreenConstraints: false,
                ),
              ),
          mobileWidget ??
              const Center(
                child: ThemedText(
                  "No Mobile screen for this page :(",
                  showThemed: true,
                  useScreenConstraints: false,
                ),
              )
        ],
      );
    });
  }
}
