// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:provider/provider.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class BorderLabel extends StatelessWidget {
  final Widget child;
  final String label;
  final double? height;
  final double? width;
  final VoidCallback? onPressed;
  final VoidCallback? onCopied;
  final VoidCallback? onClear;
  final String? tip;

  final List<Widget> actions;

  const BorderLabel({
    Key? key,
    required this.label,
    required this.child,
    this.actions = const [],
    this.onPressed,
    this.onCopied,
    this.onClear,
    this.tip,
    this.height,
    this.width,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Consumer<VertticeUIState>(builder: (context, value, _) {
      return Container(
        margin: const EdgeInsets.symmetric(vertical: 6),
        child: LayoutBuilder(
          builder: (context, constraints) {
            final double finalizedWidth = width ??
                (constraints.maxWidth != double.infinity
                    ? constraints.maxWidth
                    : 400);
            final double finalizedHeight = height ?? 100;
            double padding = 1.5;
            return GestureDetector(
              onTap: onPressed,
              child: Container(
                width: finalizedWidth,
                height: finalizedHeight,
                decoration: const BoxDecoration(
                  borderRadius: BorderRadius.all(
                    Radius.circular(
                      5,
                    ),
                  ),
                ),
                child: Container(
                  width: constraints.maxWidth,
                  decoration: BoxDecoration(
                    border: Border.all(
                      width: .3,
                      color: value.isDarkTheme
                          ? Colors.white70
                          : Colors.grey[850]!,
                    ),
                    borderRadius: const BorderRadius.all(
                      Radius.circular(5),
                    ),
                  ),
                  padding: EdgeInsets.all(padding),
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.center,
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: [
                      SizedBox(
                        width: finalizedWidth,
                        height: 28.5,
                        child: Column(
                          children: [
                            const SizedBox(
                              height: 1,
                            ),
                            SizedBox(
                              width: finalizedWidth,
                              child: Row(
                                crossAxisAlignment: CrossAxisAlignment.center,
                                mainAxisAlignment: MainAxisAlignment.center,
                                children: [
                                  Expanded(
                                    child: Padding(
                                      padding: EdgeInsets.all(padding),
                                      child: Align(
                                        alignment: Alignment.centerLeft,
                                        child: SelectableText(
                                          label,
                                          maxLines: 1,
                                          style: const TextStyle(
                                            fontSize: 15,
                                            overflow: TextOverflow.fade,
                                          ),
                                        ),
                                      ),
                                    ),
                                  ),
                                  ...[
                                    ...actions,
                                    if (onClear != null)
                                      ThemedPhysicalModel(
                                        color: Colors.grey,
                                        elevation: 5,
                                        shape: BoxShape.circle,
                                        child: InkWell(
                                          onTap: onClear!,
                                          child: ThemedCircleAvatar(
                                            backgroundColor: Colors.red,
                                            child: ThemedIcons.clear(
                                              iconStyle: IconStyle(
                                                color: Colors.white,
                                              ),
                                            ),
                                          ),
                                        ),
                                      ),
                                    if (onCopied != null)
                                      ThemedPhysicalModel(
                                        color: Colors.grey,
                                        elevation: 5,
                                        shape: BoxShape.circle,
                                        child: InkWell(
                                          onTap: onCopied!,
                                          child: ThemedCircleAvatar(
                                            child: ThemedIcons.copy(
                                              iconStyle: IconStyle(
                                                size: 20,
                                              ),
                                            ),
                                          ),
                                        ),
                                      ),
                                    if (onPressed != null)
                                      ThemedPhysicalModel(
                                        color: Colors.grey,
                                        elevation: 5,
                                        shape: BoxShape.circle,
                                        child: InkWell(
                                          onTap: onPressed!,
                                          child: ThemedCircleAvatar(
                                            child: ThemedIcons.moreHoriz(
                                              iconStyle: IconStyle(
                                                size: 20,
                                              ),
                                            ),
                                          ),
                                        ),
                                      ),
                                    if (tip != null)
                                      ThemedPhysicalModel(
                                        color: Colors.grey,
                                        elevation: 5,
                                        shape: BoxShape.circle,
                                        child: Tooltip(
                                          message: "Click to see the tip",
                                          child: InkWell(
                                            onTap: () async {
                                              await infoDialog(
                                                context: context,
                                                content: "💡 TIP: $tip",
                                              );
                                            },
                                            child: ThemedCircleAvatar(
                                              backgroundColor: Colors.grey,
                                              child: ThemedIcons.info(
                                                iconStyle: IconStyle(
                                                  color: Colors.white,
                                                ),
                                              ),
                                            ),
                                          ),
                                        ),
                                      ),
                                  ]
                                      .map(
                                        (e) => SizedBox(
                                          height: 26.5,
                                          width: 26.5,
                                          child: Center(
                                            child: SizedBox(
                                              height: 26,
                                              width: 26,
                                              child: Center(child: e),
                                            ),
                                          ),
                                        ),
                                      )
                                      .toList(),
                                ],
                              ),
                            ),
                            const SizedBox(
                              height: 1,
                            ),
                          ],
                        ),
                      ),
                      Divider(
                        height: 1,
                        color: value.isDarkTheme
                            ? Colors.white70
                            : Colors.grey[850]!,
                      ),
                      Expanded(
                        child: Padding(
                          padding: EdgeInsets.all(padding),
                          child: Align(
                            alignment: Alignment.centerLeft,
                            child: child,
                          ),
                        ),
                      ),
                    ],
                  ),
                ),
              ),
            );
          },
        ),
      );
    });
  }
}
