// 🌎 Project imports:
import '../data_factory_index.dart';

FormItem formItemToReadonlyItem({
  required FormItem formItem,
}) {
  if (formItem.field is CustomFF) {
    return formItem;
  } else if (formItem.field is TextFF ||
      formItem.field is ReadonlyTextFF ||
      formItem.field is TextBoxFF ||
      formItem.field is SelectFF ||
      formItem.field is RadioGroupFF ||
      formItem.field is ExpandSelectFF) {
    return FormItem(
      field: ReadonlyTextFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        dataBuilder: formItem.field.dataBuilder as String? Function(
            Map<String, dynamic>)?,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is PhoneNumberFF ||
      formItem.field is ReadonlyPhoneNumberFF) {
    return FormItem(
      field: ReadonlyPhoneNumberFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        dataBuilder: formItem.field.dataBuilder as Map<String, dynamic>?
            Function(Map<String, dynamic>)?,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is AddressOnMapFF ||
      formItem.field is ReadonlyAddressOnMapFF) {
    return FormItem(
      field: ReadonlyAddressOnMapFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        dataBuilder: formItem.field.dataBuilder as Map<String, dynamic>?
            Function(Map<String, dynamic>)?,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is ReadonlyTextListFF ||
      formItem.field is ExpandMultiSelectFF ||
      formItem.field is TextListFF ||
      formItem.field is EmailListFF) {
    return FormItem(
      field: ReadonlyTextListFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        dataBuilder: formItem.field.dataBuilder as List<String>? Function(
            Map<String, dynamic>)?,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is PhoneNumberListFF ||
      formItem.field is ReadonlyPhoneNumberListFF) {
    return FormItem(
      field: ReadonlyPhoneNumberListFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        dataBuilder: formItem.field.dataBuilder as List<Map<String, dynamic>>?
            Function(Map<String, dynamic>)?,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is ReadonlyNumberRangeFF ||
      formItem.field is NumberRangeFF) {
    return FormItem(
      field: ReadonlyNumberRangeFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        dataBuilder: formItem.field.dataBuilder as Map<String, num?>? Function(
            Map<String, dynamic>)?,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is IntegerFF ||
      formItem.field is DoubleFF ||
      formItem.field is ReadonlyNumberFF) {
    return FormItem(
      field: ReadonlyNumberFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        dataBuilder:
            formItem.field.dataBuilder as num? Function(Map<String, dynamic>)?,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is ReadonlyDateTimeFF) {
    return FormItem(
      field: ReadonlyDateTimeFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        dataBuilder:
            formItem.field.dataBuilder as int? Function(Map<String, dynamic>)?,
      ),
    );
  } else if (formItem.field is DateTimeRangeFF ||
      formItem.field is ReadonlyDateTimeRangeFF) {
    return FormItem(
      field: ReadonlyDateTimeRangeFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        dataBuilder: formItem.field.dataBuilder as Map<String, int?>? Function(
            Map<String, dynamic>)?,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is DateRangeFF ||
      formItem.field is ReadonlyDateRangeFF) {
    return FormItem(
      field: ReadonlyDateRangeFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        dataBuilder: formItem.field.dataBuilder as Map<String, int?>? Function(
            Map<String, dynamic>)?,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is ReadonlyDateFF || formItem.field is DateFF) {
    return FormItem(
      field: ReadonlyDateFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        dataBuilder:
            formItem.field.dataBuilder as int? Function(Map<String, dynamic>)?,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is TimeRangeFF) {
    return FormItem(
      field: ReadonlyTimeRangeFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        dataBuilder: formItem.field.dataBuilder as Map<String, int?>? Function(
            Map<String, dynamic>)?,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is TimeFF || formItem.field is ReadonlyTimeFF) {
    return FormItem(
      field: ReadonlyTimeFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        dataBuilder:
            formItem.field.dataBuilder as int? Function(Map<String, dynamic>)?,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is ColorSelectorFF) {
    return FormItem(
      field: ColorSelectorFF(
        readOnly: true,
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        colors: (formItem.field as ColorSelectorFF).colors,
        dataBuilder: formItem.field.dataBuilder as SelectorColors? Function(
            Map<String, dynamic>)?,
        searchPath: formItem.field.searchPath,
      ),
    );
  }
  return FormItem(
    field: ReadonlyTextFF(
      displayName: formItem.field.displayName,
      dataBuilder:
          formItem.field.dataBuilder as String? Function(Map<String, dynamic>)?,
      name: formItem.field.name,
      searchPath: formItem.field.searchPath,
    ),
  );
}

FormItem formItemToFilterItem({required FormItem formItem}) {
  if (formItem.field is TextFF ||
      formItem.field is AddressOnMapFF ||
      formItem.field is ReadonlyTextFF ||
      formItem.field is TextListFF ||
      formItem.field is EmailListFF ||
      formItem.field is TextBoxFF ||
      formItem.field is PhoneNumberListFF ||
      formItem.field is PhoneNumberFF ||
      formItem.field is ReadonlyPhoneNumberFF) {
    return FormItem(
      field: TextFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is IntegerFF ||
      formItem.field is NumberRangeFF ||
      formItem.field is ReadonlyNumberFF ||
      formItem.field is DoubleFF) {
    return FormItem(
      field: NumberRangeFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is SelectFF) {
    return FormItem(
        field: SelectFF(
      displayName: formItem.field.displayName,
      name: formItem.field.name,
      options: (formItem.field as SelectFF).options,
      searchPath: formItem.field.searchPath,
    ));
  } else if (formItem.field is RadioGroupFF) {
    return FormItem(
      field: RadioGroupFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        options: (formItem.field as RadioGroupFF).options,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is ExpandSelectFF) {
    return FormItem(
      field: ExpandSelectFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        optionsID: (formItem.field as ExpandSelectFF).optionsID,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is ExpandMultiSelectFF) {
    return FormItem(
      field: ExpandMultiSelectFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        optionsID: (formItem.field as ExpandMultiSelectFF).optionsID,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is DateRangeFF ||
      formItem.field is ReadonlyDateFF ||
      formItem.field is ReadonlyDateTimeFF ||
      formItem.field is DateFF ||
      formItem.field is ReadonlyDateRangeFF ||
      formItem.field is DateTimeRangeFF ||
      formItem.field is ReadonlyDateTimeRangeFF) {
    return FormItem(
      field: DateRangeFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is TimeFF ||
      formItem.field is ReadonlyTimeFF ||
      formItem.field is TimeRangeFF) {
    return FormItem(
      field: TimeRangeFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        searchPath: formItem.field.searchPath,
      ),
    );
  } else if (formItem.field is ColorSelectorFF) {
    return FormItem(
      field: ColorSelectorFF(
        displayName: formItem.field.displayName,
        name: formItem.field.name,
        colors: (formItem.field as ColorSelectorFF).colors,
        searchPath: formItem.field.searchPath,
      ),
    );
  }
  return FormItem(
    field: TextFF(
      displayName: formItem.field.displayName,
      name: formItem.field.name,
      searchPath: formItem.field.searchPath,
    ),
  );
}

List<FormItem> formItemsToFilterItems({required List<FormItem> formItems}) {
  return formItems
      .map((formItem) => formItemToFilterItem(formItem: formItem))
      .toList();
}
