// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:verttice_logic/verttice_logic.dart';

// 🌎 Project imports:
import 'package:verttice_ui/src/widgets/border_label.dart';

class CheckboxFormFactory extends StatefulWidget {
  final Function(String name, dynamic value) onChanged;
  final String displayName;
  final String name;
  final bool? value;
  final FormFieldState<dynamic>? state;
  final String? tip;

  const CheckboxFormFactory({
    Key? key,
    required this.onChanged,
    required this.displayName,
    required this.name,
    this.value,
    this.state,
    this.tip,
  }) : super(key: key);

  @override
  State<CheckboxFormFactory> createState() => _CheckboxFormFactoryState();
}

class _CheckboxFormFactoryState extends State<CheckboxFormFactory> {
  bool _value = false;

  @override
  void initState() {
    _value = DataClassUtils.dynamicToBool(widget.value, defaultValue: false);
    super.initState();
  }

  @override
  void didUpdateWidget(covariant CheckboxFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = DataClassUtils.dynamicToBool(widget.value, defaultValue: false);
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName.length > 30
          ? "${widget.displayName.substring(0, 30)}..."
          : widget.displayName,
      tip: widget.tip,
      child: Row(
        children: [
          Expanded(
            child: SelectableText(
              widget.displayName,
              maxLines: 3,
              style:
                  const TextStyle(fontSize: 12.0, overflow: TextOverflow.clip),
            ),
          ),
          Checkbox(
            value: _value,
            onChanged: (bool? value) {
              widget.onChanged(widget.name, value);
              widget.state?.didChange(value);
            },
          ),
        ],
      ),
    );
  }
}
