// 🐦 Flutter imports:
import "package:flutter/material.dart";
import 'package:flutter/services.dart';

// 📦 Package imports:
import 'package:verttice_logic/verttice_logic.dart';

// 🌎 Project imports:
import "package:verttice_ui/src/const/icons.dart";
import 'package:verttice_ui/src/widgets/border_label.dart';
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';

class DateFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final dynamic value;
  final int? minDate;
  final int? maxDate;
  final Function(String name, int? value) onChanged;
  final FormFieldState<dynamic>? state;
  final String? tip;

  const DateFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.value,
    required this.onChanged,
    this.minDate,
    this.maxDate,
    this.state,
    this.tip,
  }) : super(key: key);

  @override
  State<DateFormFactory> createState() => _DateFormState();
}

class _DateFormState extends State<DateFormFactory> {
  int _value = 0;
  int? _maxDate;
  int? _minDate;

  @override
  void initState() {
    super.initState();
    _maxDate = widget.maxDate;
    _minDate = widget.minDate;
    _value = DataClassUtils.dynamicToInt(widget.value, defaultValue: 0)!;
  }

  @override
  void didUpdateWidget(covariant DateFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value ||
        _minDate != widget.minDate ||
        _maxDate != widget.maxDate) {
      _maxDate = widget.maxDate;
      _minDate = widget.minDate;
      _value = DataClassUtils.dynamicToInt(widget.value, defaultValue: 0)!;
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      tip: widget.tip,
      onCopied: () async {
        await Clipboard.setData(
          ClipboardData(
            text: _value != 0
                ? Dates.formatDataAndTime(
                    milliseconds: _value,
                    time: false,
                  )
                : "",
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Copied to clipboard",
        );
      },
      onClear: () {
        widget.onChanged(widget.name, 0);
        widget.state?.didChange(0);
      },
      child: ThemedCleanTextButton(
        _value != 0
            ? Dates.formatDataAndTime(
                milliseconds: _value,
                time: false,
              )
            : "",
        icon: ThemedIcons.calendar(),
        onPressed: () async {
          DateTime? date = await showDatePicker(
            context: context,
            initialDate: DateTime.fromMillisecondsSinceEpoch(
              _value != 0 ? _value : DateTime.now().millisecondsSinceEpoch,
            ),
            firstDate: _minDate != null
                ? DateTime.fromMillisecondsSinceEpoch(_minDate!)
                : DateTime(1950),
            lastDate: _maxDate != null
                ? DateTime.fromMillisecondsSinceEpoch(_maxDate!)
                : DateTime(2100),
          );

          if (date == null) {
            return;
          }
          final utcDate = date.toUtc();
          widget.onChanged(widget.name, utcDate.millisecondsSinceEpoch);
          widget.state?.didChange(utcDate.millisecondsSinceEpoch);
        },
      ),
    );
  }
}
