// 🎯 Dart imports:
import 'dart:math';

// 🐦 Flutter imports:
import "package:flutter/material.dart";
import 'package:flutter/services.dart';

// 📦 Package imports:
import "package:collection/collection.dart";
import 'package:syncfusion_flutter_datepicker/datepicker.dart';
import 'package:verttice_logic/verttice_logic.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class DateRangeFormFactory extends StatefulWidget {
  final Function(String name, Map<String, int>? value) onChanged;
  final String displayName;
  final String name;
  final dynamic value;
  final FormFieldState<dynamic>? state;
  final String? tip;

  const DateRangeFormFactory({
    Key? key,
    required this.onChanged,
    required this.displayName,
    required this.name,
    required this.value,
    this.state,
    this.tip,
  }) : super(key: key);

  @override
  State<DateRangeFormFactory> createState() => _DateRangeFormState();
}

class _DateRangeFormState extends State<DateRangeFormFactory> {
  var _value = <String, int?>{};

  @override
  void initState() {
    super.initState();
    _value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value)
        .cast<String, int?>();
  }

  @override
  void didUpdateWidget(covariant DateRangeFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    final value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value);
    if (!const MapEquality().equals(_value, value)) {
      _value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value)
          .cast<String, int?>();
    }
  }

  @override
  Widget build(BuildContext context) {
    final label =
        "${_value["startDate"] != null && _value["startDate"] is int ? Dates.formatDataAndTime(milliseconds: _value["startDate"]!, time: false) : ""} - ${_value["endDate"] != null && _value["endDate"] is int ? Dates.formatDataAndTime(milliseconds: _value["endDate"]!, time: false) : ""}";
    return BorderLabel(
      label: widget.displayName,
      tip: widget.tip ??
          'Select date range\n if you want to select the same date for start and end click on the date twice',
      onCopied: () async {
        await Clipboard.setData(
          ClipboardData(
            text:
                "${_value["startDate"] != null && _value["startDate"] is int ? Dates.formatDataAndTime(milliseconds: _value["startDate"]!, time: false) : ""} - ${_value["endDate"] != null && _value["endDate"] is int ? Dates.formatDataAndTime(milliseconds: _value["endDate"]!, time: false) : ""}",
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Copied to clipboard",
        );
      },
      onClear: () {
        widget.onChanged(widget.name, null);
        widget.state?.didChange(null);
      },
      child: Tooltip(
        message: label,
        child: ThemedCleanTextButton(
          label,
          icon: ThemedIcons.calendar(),
          onPressed: () async {
            DateTime? startDate;
            DateTime? endDate;
            if (_value['startDate'] != null &&
                _value['startDate'] is int &&
                _value['startDate']! > 0) {
              startDate = DateTime.fromMillisecondsSinceEpoch(
                _value['startDate']!,
              );
            }

            if (_value['startDate'] != null &&
                _value['endDate'] is int &&
                _value['endDate']! > 0) {
              endDate = DateTime.fromMillisecondsSinceEpoch(
                _value['endDate']!,
              );
            }
            final PickerDateRange range = PickerDateRange(startDate, endDate);

            await sizedDialog(
              context: context,
              height: MediaQuery.of(context).size.height * .8,
              width: min(MediaQuery.of(context).size.width * .8, 600),
              appBarBuilder: (context) => const ThemedAppBar(
                navigateBackButton: true,
                title: 'Select Date Range',
              ),
              builder: (context) => Center(
                child: SfDateRangePicker(
                  view: DateRangePickerView.month,
                  selectionMode: DateRangePickerSelectionMode.range,
                  initialSelectedRange: range,
                  minDate: DateTime(1950),
                  maxDate: DateTime(2100),
                  showActionButtons: true,
                  showNavigationArrow: true,
                  showTodayButton: true,
                  allowViewNavigation: true,
                  onCancel: () {
                    Navigator.pop(context);
                  },
                  onSubmit: (dynamic data) {
                    if (data is PickerDateRange) {
                      final date = data;

                      final dateStart = date.startDate;
                      if (dateStart == null) {
                        Navigator.pop(context);
                        return;
                      }
                      final dateEnd = date.endDate ?? dateStart;

                      final chosenDate = {
                        "startDate": dateStart.toUtc().millisecondsSinceEpoch,
                        "endDate": DateTime(
                          dateEnd.year,
                          dateEnd.month,
                          dateEnd.day,
                          23,
                          59,
                        ).toUtc().millisecondsSinceEpoch,
                      };
                      widget.onChanged(widget.name, chosenDate);
                      widget.state?.didChange(chosenDate);
                    }
                    Navigator.pop(context);
                  },
                ),
              ),
            );
          },
        ),
      ),
    );
  }
}
