// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class ExpandSelectAddOption extends StatefulWidget {
  final String displayName;
  final String optionsID;
  final VoidCallback onDone;
  final List<String> currentOptions;
  const ExpandSelectAddOption({
    Key? key,
    required this.displayName,
    required this.optionsID,
    required this.onDone,
    required this.currentOptions,
  }) : super(key: key);

  @override
  State<ExpandSelectAddOption> createState() => _ExpandSelectAddOptionState();
}

class _ExpandSelectAddOptionState extends State<ExpandSelectAddOption> {
  String _newOption = '';
  bool _savingNewOption = false;

  _addOption(BuildContext context) async {
    try {
      if (widget.currentOptions
          .map((e) => e.toLowerCase())
          .contains(_newOption.toLowerCase())) {
        ThemedToast.warning(
          context: context,
          message: "Option already exist",
        );
        return;
      }
      await VertticeUIConfig.instance.expandableOptionsConfig.setOption(
        optionsID: widget.optionsID,
        option: _newOption,
      );
    } catch (e) {
      ThemedToast.error(context: context, message: "Failed to add option");
    } finally {
      widget.onDone();
      setState(() {
        _savingNewOption = false;
        _newOption = "";
      });
      Navigator.of(context).pop();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        ThemedAppBar(
          navigateBackButton: true,
          title: "Add option to ${widget.displayName}",
        ),
        Expanded(
          child: Padding(
            padding: const EdgeInsets.all(10.0),
            child: Column(
              children: [
                SizedBox(
                  height: 150,
                  width: 430,
                  child: TextFormFactory(
                    value: _newOption,
                    displayName: "New Option",
                    name: "option",
                    onChanged: (name, data) {
                      setState(() {
                        _newOption = data;
                      });
                    },
                  ),
                ),
                sizedBoxH16,
                ThemedTextButton(
                  "Save option",
                  showLoading: _savingNewOption,
                  onPressed: () async {
                    if (_newOption == "") {
                      return;
                    }

                    setState(() {
                      _savingNewOption = true;
                    });
                    await _addOption(context);
                  },
                )
              ],
            ),
          ),
        ),
      ],
    );
  }
}
