// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:collection/collection.dart';
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class FilesFormFactory extends StatelessWidget {
  final String displayName;
  final String name;
  final dynamic value;
  final bool showDeleteFile;
  final Function(String name, List<Map<String, dynamic>> value) onChanged;
  final FormFieldState<dynamic>? state;
  final String path;
  final String? tip;

  // by default it will upload files via the verttice cofig global function
  final Future<FileModel> Function({
    required BuildContext context,
    required List<UploadFileDataModel> files,
  })? altUploadImageFun;

  const FilesFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.onChanged,
    required this.path,
    this.showDeleteFile = false,
    this.tip,
    this.value,
    this.state,
    this.altUploadImageFun,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final List<FileModel> filesToDisplay = value is List
        ? (value as List)
            .map((e) =>
                FileModel.fromMap(DataClassUtils.dynamicToMapStringDynamic(e)))
            .where((element) => element.url != null)
            .toList()
        : [];
    return BorderLabel(
      label: displayName,
      tip: tip,
      onPressed: () async {
        Navigator.push(
          context,
          MaterialPageRoute(
            builder: (context) => UplaodFilesAction(
              altUploadFilesFun: altUploadImageFun == null
                  ? null
                  : ({
                      required BuildContext context,
                      required List<UploadFileDataModel> files,
                    }) async {
                      final res = await altUploadImageFun!(
                        context: context,
                        files: files,
                      );
                      return [res];
                    },
              path: path,
              filesDataCallback: ({
                required List<FileModel> files,
              }) async {
                onChanged(
                    name,
                    [...filesToDisplay, ...files]
                        .map((e) => e.toMap())
                        .toList());
                state?.didChange([...filesToDisplay, ...files]
                    .map((e) => e.toMap())
                    .toList());
              },
              multi: true,
            ),
          ),
        );
      },
      child: filesToDisplay.isEmpty
          ? const ThemedText('No files yet :(')
          : ListView(
              scrollDirection: Axis.horizontal,
              children: filesToDisplay
                  .mapIndexed(
                    (int index, file) => ConstrainedBox(
                      constraints: const BoxConstraints(minWidth: 60),
                      child: Padding(
                        padding: const EdgeInsets.symmetric(horizontal: 5),
                        child: FileItem(
                          width: 130,
                          deleteFileCallback: showDeleteFile
                              ? (FileModel file) {
                                  final filesToReturn = filesToDisplay;
                                  filesToReturn.removeAt(index);
                                  onChanged(
                                    name,
                                    filesToReturn
                                        .map((e) => e.toMap())
                                        .toList(),
                                  );
                                  state?.didChange(filesToReturn
                                      .map((e) => e.toMap())
                                      .toList());
                                }
                              : null,
                          file: file,
                        ),
                      ),
                    ),
                  )
                  .toList(),
            ),
    );
  }
}
