// 🎯 Dart imports:
import "dart:math";

// 🐦 Flutter imports:
import "package:flutter/material.dart";
import "package:flutter/services.dart";

// 📦 Package imports:
import "package:collection/collection.dart";
import 'package:verttice_logic/verttice_logic.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class NumberRangeFormFactory extends StatefulWidget {
  final Function(String name, Map<String, double?> value) onChanged;
  final String displayName;
  final String name;
  final Map<String, double?>? value;
  final FormFieldState<dynamic>? state;
  final String? tip;

  const NumberRangeFormFactory({
    Key? key,
    required this.onChanged,
    required this.displayName,
    required this.name,
    required this.value,
    this.state,
    this.tip,
  }) : super(key: key);

  @override
  State<NumberRangeFormFactory> createState() => _NumberRangeFormState();
}

class _NumberRangeFormState extends State<NumberRangeFormFactory> {
  var _value = <String, double?>{};

  @override
  void initState() {
    _value = widget.value ??
        {
          "lowNumber": null,
          "heighNumber": null,
        };
    super.initState();
  }

  @override
  void didUpdateWidget(covariant NumberRangeFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const MapEquality().equals(_value, widget.value)) {
      _value = widget.value ??
          {
            "lowNumber": null,
            "heighNumber": null,
          };
    }
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(builder: (context, constraints) {
      return BorderLabel(
        label: widget.displayName,
        tip: widget.tip,
        child: Row(
          crossAxisAlignment: CrossAxisAlignment.center,
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            SizedBox(
              width: (constraints.maxWidth / 2) - 12.5,
              child: TextFormFactory(
                showStandardBorder: false,
                displayName: "Bigger Than Or Equal To",
                name: "lowNumber",
                type: TextFieldInputFF.double,
                onChanged: (String name, dynamic val) {
                  if (val is double || val == null) {
                    setState(() {
                      _value = {
                        ..._value,
                        name: val == null ? val : val as double
                      };
                      widget.onChanged(widget.name, _value);
                      widget.state?.didChange(_value);
                    });
                  }
                },
                value: DataClassUtils.dynamicToString(
                  _value["lowNumber"],
                  defaultValue: "",
                ),
              ),
            ),
            SizedBox(
              width: (constraints.maxWidth / 2) - 12.5,
              child: TextFormFactory(
                showStandardBorder: false,
                displayName: "Less Than Or Equal To",
                name: "heighNumber",
                type: TextFieldInputFF.double,
                onChanged: (String name, dynamic val) {
                  if (val is double || val == null) {
                    setState(() {
                      _value = {..._value, name: val};
                      widget.onChanged(widget.name, _value);
                      widget.state?.didChange(_value);
                    });
                  }
                },
                value: DataClassUtils.dynamicToString(
                  _value["heighNumber"],
                  defaultValue: "",
                ),
              ),
            ),
          ],
        ),
      );
    });
  }
}

class NumberRangeInput extends StatefulWidget {
  final String displayName;
  final String name;
  final dynamic value;

  final Function(String name, dynamic value) onChanged;

  const NumberRangeInput({
    Key? key,
    required this.displayName,
    required this.name,
    required this.onChanged,
    this.value,
  }) : super(key: key);
  @override
  State<NumberRangeInput> createState() => _TextFormState();
}

class _TextFormState extends State<NumberRangeInput> {
  TextEditingController? _controller;
  String _value = "";
  int _baseOffset = 0;
  late FocusNode _focusNode;
  @override
  void initState() {
    super.initState();

    _value = DataClassUtils.dynamicToDouble(widget.value).toString();
    _controller = TextEditingController(
      text: _value,
    );

    _controller!.selection = TextSelection.fromPosition(
      TextPosition(
        offset: min(
          _baseOffset,
          _controller != null ? _controller!.text.length : 0,
        ),
      ),
    );

    _controller!.addListener(_listen);
    _focusNode = FocusNode();
  }

  @override
  void dispose() {
    _focusNode.dispose();
    _controller!.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant NumberRangeInput oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = DataClassUtils.dynamicToDouble(widget.value).toString();

      _controller = TextEditingController(
        text: _value,
      );
      _controller!.selection = TextSelection.fromPosition(
        TextPosition(
          offset: min(
            _baseOffset,
            _controller != null ? _controller!.text.length : 0,
          ),
        ),
      );
      _controller!.addListener(_listen);
    }
  }

  void _listen() {
    setState(() {
      _baseOffset = _controller!.selection.baseOffset;
    });
  }

  @override
  Widget build(BuildContext context) {
    TextInputType inputType = TextInputType.number;

    return LayoutBuilder(builder: (context, constraints) {
      return SizedBox(
        width: constraints.maxWidth,
        child: MouseRegion(
          onEnter: (_) {
            setState(() {
              _focusNode.requestFocus();
            });
          },
          onExit: (_) {
            setState(() {
              _focusNode.unfocus();
            });
          },
          child: TextFormField(
            focusNode: _focusNode,
            cursorColor: Colors.amberAccent,
            onChanged: (val) {
              setState(() {
                _value = val;
              });
              final newVal = val != "" ? double.parse(val) : null;
              widget.onChanged(widget.name, newVal);
            },
            controller: _controller,
            keyboardType: inputType,
            inputFormatters: <TextInputFormatter>[
              FilteringTextInputFormatter.allow(RegExp(r"(^\-?\d*\.?\d*)")),
            ],
            textDirection: TextDirection.ltr,
            autofocus: false,
            decoration: InputDecoration(
              label: Text(widget.displayName),
              suffixIcon: ThemedIconButton(
                onPressed: () {
                  _controller!.clear();
                  widget.onChanged(widget.name, "");
                },
                icon: ThemedIcons.clear(
                  iconStyle: IconStyle(
                    color: Colors.redAccent,
                  ),
                ),
              ),
            ),
          ),
        ),
      );
    });
  }
}
