// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:verttice_logic/verttice_logic.dart';

// 🌎 Project imports:
import 'package:verttice_ui/src/widgets/border_label.dart';

enum ReadonlyFormType {
  number,
  text,
  time,
  phoneNumber,
  date,
  dateTime,
  dateRange,
  dateTimeRange,
  numberRange,
  timeRange,
  textList,
  phoneNumberList,
  addressOnMap,
}

class ReadonlyFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final dynamic value;
  final ReadonlyFormType type;
  final String? tip;

  const ReadonlyFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.type,
    this.value,
    this.tip,
  }) : super(key: key);

  @override
  State<ReadonlyFormFactory> createState() => _ReadonlyState();
}

class _ReadonlyState extends State<ReadonlyFormFactory> {
  dynamic _value;

  @override
  void initState() {
    super.initState();
    _value = widget.value;
  }

  @override
  void didUpdateWidget(covariant ReadonlyFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = widget.value;
    }
  }

  @override
  Widget build(BuildContext context) {
    String value = '';
    if (_value == null) {
      value = '';
    } else if (widget.type == ReadonlyFormType.textList) {
      value = _value == null || _value is! List
          ? ""
          : DataClassUtils.dynamicToStringList(_value).join(", ");
    } else if (widget.type == ReadonlyFormType.phoneNumberList) {
      value = _value == null || _value is! List
          ? ""
          : (_value as List)
              .map((e) => _value == null || _value is! Map
                  ? ""
                  : _value['prettyNumber']?.toString() ?? '')
              .join(", ");
    } else if (widget.type == ReadonlyFormType.phoneNumber) {
      value = _value == null || _value is! Map
          ? ""
          : _value['prettyNumber']?.toString() ?? '';
    } else if (widget.type == ReadonlyFormType.addressOnMap) {
      value = _value == null || _value is! Map
          ? ""
          : _value['fullAddress']?.toString() ?? '';
    } else if (widget.type == ReadonlyFormType.number) {
      value = DataClassUtils.dynamicToDouble(_value, defaultValue: 0)!
          .toStringAsFixed(2);
    } else if (widget.type == ReadonlyFormType.dateRange) {
      value = _value == null || _value is! Map
          ? ""
          : "${_value["startDate"] != null && _value["startDate"] is int ? Dates.formatDataAndTime(milliseconds: DataClassUtils.dynamicToInt(_value["startDate"], defaultValue: 0)!, time: false) : ""} - ${_value["endDate"] != null && _value["endDate"] is int ? Dates.formatDataAndTime(milliseconds: DataClassUtils.dynamicToInt(_value["endDate"], defaultValue: 0)!, time: false) : ""}";
    } else if (widget.type == ReadonlyFormType.dateTimeRange) {
      value = _value == null || _value is! Map
          ? ""
          : "${_value["startDate"] != null && _value["startDate"] is int ? Dates.formatDataAndTime(milliseconds: DataClassUtils.dynamicToInt(_value["startDate"], defaultValue: 0)!) : ""} - ${_value["endDate"] != null && _value["endDate"] is int ? Dates.formatDataAndTime(milliseconds: DataClassUtils.dynamicToInt(_value["endDate"], defaultValue: 0)!) : ""}";
    } else if (widget.type == ReadonlyFormType.timeRange) {
      value = _value == null || _value is! Map
          ? ""
          : "${_value["startTime"] != null && _value["startTime"] is int ? Dates.formatDataAndTime(
              milliseconds: DataClassUtils.dynamicToInt(
                _value["startTime"],
                defaultValue: 0,
              )!,
              date: false,
            ) : ""} - ${_value["endTime"] != null && _value["endTime"] is int ? Dates.formatDataAndTime(
              milliseconds: DataClassUtils.dynamicToInt(
                _value["endTime"],
                defaultValue: 0,
              )!,
              date: false,
            ) : ""}";
    } else if (widget.type == ReadonlyFormType.text) {
      value = (_value?.toString() ?? "");
    } else if (widget.type == ReadonlyFormType.date) {
      value = Dates.formatDataAndTime(
        milliseconds: DataClassUtils.dynamicToInt(_value ?? 0)!,
        time: false,
      );
    } else if (widget.type == ReadonlyFormType.time) {
      value = Dates.formatDataAndTime(
        milliseconds: DataClassUtils.dynamicToInt(_value ?? 0)!,
        date: false,
      );
    } else if (widget.type == ReadonlyFormType.dateTime) {
      value = Dates.formatDataAndTime(
        milliseconds: DataClassUtils.dynamicToInt(_value ?? 0)!,
      );
    } else if (widget.type == ReadonlyFormType.numberRange) {
      value = _value == null || _value is! Map
          ? ""
          : "${_value["lowNumber"] != null && _value["lowNumber"] is num ? (_value["lowNumber"] as num).toStringAsFixed(2) : ""} - ${_value["heighNumber"] != null && _value["heighNumber"] is num ? (_value["heighNumber"] as num).toStringAsFixed(2) : ""}";
    }
    return BorderLabel(
      label: widget.displayName,
      tip: widget.tip,
      child: Builder(
        builder: (BuildContext context) {
          return SelectableText(value);
        },
      ),
    );
  }
}
