// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import "package:collection/collection.dart";

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class SelectFormFactory<T> extends StatefulWidget {
  final Function(String name, T? value) onChanged;
  final String displayName;
  final String name;
  final List<T> options;
  final String Function(dynamic)? labelsBuilder;
  final dynamic value;
  final FormFieldState<T>? state;
  final String? tip;

  const SelectFormFactory({
    Key? key,
    required this.onChanged,
    required this.displayName,
    required this.name,
    required this.options,
    this.labelsBuilder,
    this.value,
    this.state,
    this.tip,
  }) : super(key: key);

  @override
  State<SelectFormFactory> createState() => _SelectFormState<T>();
}

class _SelectFormState<T> extends State<SelectFormFactory<T>> {
  late dynamic _value;

  @override
  void initState() {
    _value = widget.value;
    super.initState();
  }

  @override
  void didUpdateWidget(covariant SelectFormFactory<T> oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const DeepCollectionEquality.unordered()
        .equals(_value, widget.value)) {
      _value = widget.value;
    }
  }

  @override
  Widget build(BuildContext context) {
    final Set<T> items = {};
    items.addAll(widget.options.where((element) => element != null));
    if (_value != null && _value is T && !items.toList().contains(_value)) {
      items.add(_value);
    }
    String label = _value?.toString() ?? '';
    if (_value != null && widget.labelsBuilder != null) {
      label = widget.labelsBuilder!(_value);
    }
    return BorderLabel(
      label: widget.displayName,
      tip: widget.tip,
      onClear: () {
        widget.onChanged(widget.name, null);
        widget.state?.didChange(null);
      },
      child: SelectableText(label),
      onPressed: () async {
        await sizedDialog(
          context: context,
          height: 390,
          width: 400,
          appBarBuilder: (context) => ThemedAppBar(
            navigateBackButton: true,
            title: 'Select ${widget.displayName}',
          ),
          builder: (context) => ThemedSearchList<T>(
            items: items.toList(),
            onSelectedItem: (item) async {
              widget.onChanged(widget.name, item);
              widget.state?.didChange(item);
              Navigator.pop(context);
            },
            itemStringLabelBuilder: (item) {
              if (widget.labelsBuilder != null) {
                return widget.labelsBuilder!(item);
              }

              return item?.toString() ?? "";
            },
          ),
        );
      },
    );
  }
}
