// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class SignPadFormFactory extends StatelessWidget {
  final String displayName;
  final String name;
  final dynamic value;
  final Function(String name, Map<String, dynamic> value) onChanged;
  final FormFieldState<dynamic>? state;
  final String path;
  final String? tip;

  // by default it will upload files via the verttice cofig global function
  final Future<FileModel> Function({
    required BuildContext context,
    required UploadFileDataModel file,
  })? altUploadImageFun;

  const SignPadFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.onChanged,
    required this.path,
    this.tip,
    this.value,
    this.state,
    this.altUploadImageFun,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final file = value is Map
        ? FileModel.fromMap(DataClassUtils.dynamicToMapStringDynamic(value))
        : null;
    return BorderLabel(
      label: displayName,
      tip: tip,
      onPressed: () async {
        Navigator.push(
          context,
          MaterialPageRoute(
            builder: (context) => Painter(
              altUploadFilesFun: altUploadImageFun == null
                  ? null
                  : ({
                      required BuildContext context,
                      required List<UploadFileDataModel> files,
                    }) async {
                      final res = await altUploadImageFun!(
                        context: context,
                        file: files.first,
                      );
                      return [res];
                    },
              path: path,
              imageDataCallback: (
                FileModel file,
              ) async {
                onChanged(name, file.toMap());
                state?.didChange(file.toMap());
              },
            ),
          ),
        );
      },
      child: file != null && file.url != null
          ? ImageViewer(
              url: file.url,
            )
          : const ThemedText('No signature yet :('),
    );
  }
}
