// 🎯 Dart imports:
import "dart:math";

// 🐦 Flutter imports:
import "package:flutter/material.dart";
import 'package:flutter/services.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class TextBoxFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final String value;
  final FormFieldState<dynamic>? state;
  final Function(String name, dynamic value) onChanged;
  final String? templateOptionsID;
  final String? tip;

  const TextBoxFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.onChanged,
    required this.value,
    this.state,
    this.templateOptionsID,
    this.tip,
  }) : super(key: key);
  @override
  State<TextBoxFormFactory> createState() => _TextBoxFormFactoryState();
}

class _TextBoxFormFactoryState extends State<TextBoxFormFactory> {
  TextEditingController _controller = TextEditingController();
  String _value = "";
  int _baseOffset = 0;

  @override
  void initState() {
    super.initState();
    _value = widget.value;
    _controller = TextEditingController(
      text: _value,
    );
    _controller.selection = TextSelection.fromPosition(
      TextPosition(
        offset: min(
          _baseOffset,
          _controller.text.length,
        ),
      ),
    );

    _controller.addListener(_listen);
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant TextBoxFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = widget.value;
      _controller = TextEditingController(
        text: _value,
      );
      _controller.selection = TextSelection.fromPosition(
        TextPosition(
          offset: min(
            _baseOffset,
            _controller.text.length,
          ),
        ),
      );
      _controller.addListener(_listen);
    }
  }

  void _listen() {
    setState(() {
      _baseOffset = _controller.selection.baseOffset;
    });
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      tip: widget.tip,
      onCopied: () async {
        await Clipboard.setData(
          ClipboardData(
            text: _value,
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Copied to clipboard",
        );
      },
      onPressed: () async {
        await sizedDialog(
          context: context,
          height: 300,
          width: 400,
          appBarBuilder: (context) => ThemedAppBar(
            navigateBackButton: true,
            title: widget.displayName,
            actions: [
              if (widget.templateOptionsID != null)
                // SizedBox(
                //   height: 40,
                //   child: ThemedTextButton(
                //     'From Template',
                //     onPressed: () async {
                //       final value = await sizedDialog<String?>(
                //         context: context,
                //         height: 240,
                //         width: 390,
                //         child: ExpandSelectFormFactory(
                //           onChanged: (name, value) {
                //             Navigator.of(context).pop(value);
                //           },
                //           displayName: 'Templates For ${widget.displayName}',
                //           name: 'template',
                //           optionsID: widget.templateOptionsID!,
                //         ),
                //       );
                //       print(value);
                //       if (value != null) {
                //         setState(() {
                //           _value = value;
                //         });
                //         widget.onChanged(widget.name, value);
                //         widget.state?.didChange(value);
                //         if (mounted) {
                //           Navigator.pop(context);
                //         }
                //       }
                //     },
                //   ),
                // ),
                sizedBoxW8,
              ThemedIconButton(
                onPressed: (() => Navigator.pop(context)),
                icon: ThemedIcons.check(),
              ),
            ],
          ),
          builder: (context) => Padding(
            padding: const EdgeInsets.all(16),
            child: TextFormField(
              cursorColor: Colors.amberAccent,
              onChanged: (val) {
                setState(() {
                  _value = val;
                });
                widget.onChanged(widget.name, val);
                widget.state?.didChange(val);
              },
              minLines: 10,
              maxLines: 20,
              controller: _controller,
              keyboardType: TextInputType.multiline,
              textDirection: TextDirection.ltr,
              autofocus: false,
              decoration: InputDecoration(
                suffixIcon: ThemedIconButton(
                  onPressed: () {
                    _controller.clear();
                    widget.onChanged(widget.name, "");
                    widget.state?.didChange("");
                  },
                  icon: ThemedIcons.clear(
                    iconStyle: IconStyle(
                      color: Colors.redAccent,
                    ),
                  ),
                ),
              ),
            ),
          ),
        );
      },
      child: LayoutBuilder(builder: (context, constraints) {
        return SizedBox(
          width: constraints.maxWidth,
          child: Tooltip(
            message: _value,
            child: Text(
              _value,
              maxLines: 3,
              style: const TextStyle(
                overflow: TextOverflow.fade,
              ),
            ),
          ),
        );
      }),
    );
  }
}
