// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import "package:validators/validators.dart";
import 'package:verttice_logic/verttice_logic.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

enum ListType {
  text,
  email,
}

class TextListFormFactory extends StatelessWidget {
  final String displayName;
  final String name;
  final Function(String name, dynamic value) onChanged;
  final List<String>? value;
  final ListType type;
  final FormFieldState<dynamic>? state;
  final String? tip;

  const TextListFormFactory({
    Key? key,
    this.value,
    required this.displayName,
    required this.name,
    required this.onChanged,
    required this.type,
    this.state,
    this.tip,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: displayName,
      tip: tip,
      child: SelectableText(
        value?.join(", ") ?? '',
      ),
      onPressed: () async {
        await sizedDialog(
          context: context,
          height: 400,
          width: 400,
          builder: (context) => Center(
            child: _TextListFormFactoryContnet(
              displayName: displayName,
              name: name,
              onChanged: onChanged,
              type: type,
              state: state,
              value: value,
            ),
          ),
        );
      },
    );
  }
}

class _TextListFormFactoryContnet extends StatefulWidget {
  final String displayName;
  final String name;
  final Function(String name, dynamic value) onChanged;
  final List<String>? value;
  final ListType type;
  final FormFieldState<dynamic>? state;

  const _TextListFormFactoryContnet({
    Key? key,
    required this.displayName,
    required this.name,
    required this.onChanged,
    required this.type,
    this.state,
    this.value,
  }) : super(key: key);

  @override
  _TextListFormFactoryContnetState createState() =>
      _TextListFormFactoryContnetState();
}

class _TextListFormFactoryContnetState
    extends State<_TextListFormFactoryContnet> {
  List<String> _value = [];
  List<String> _newdData = [];

  @override
  void initState() {
    super.initState();
    _value = DataClassUtils.dynamicToStringList(widget.value);
    _newdData = DataClassUtils.dynamicToStringList(widget.value);
  }

  @override
  void didUpdateWidget(covariant _TextListFormFactoryContnet oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = DataClassUtils.dynamicToStringList(widget.value);
      _newdData = DataClassUtils.dynamicToStringList(widget.value);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: ThemedAppBar(
        navigateBackButton: true,
        title: widget.displayName,
        actions: [
          _AddTextFieldListItem(
            onChanged: (item) {
              setState(() {
                _newdData.add(item);
              });
            },
            type: widget.type,
          ),
          ThemedIconButton(
            onPressed: () {
              widget.onChanged(widget.name, _newdData);
              widget.state?.didChange(_newdData);
              Navigator.of(context).pop();
            },
            icon: ThemedIcons.check(),
          ),
        ],
      ),
      body: ListView.separated(
        controller: ScrollController(),
        itemCount: _newdData.length,
        physics: const BouncingScrollPhysics(),
        itemBuilder: (context, i) => _item(_newdData[i], i),
        separatorBuilder: (context, index) {
          return divider;
        },
      ),
    );
  }

  Widget _item(String item, int i) {
    return Padding(
      padding: const EdgeInsets.all(6.5),
      child: Row(
        children: [
          Expanded(
            child: _EditTextFieldListItem(
              type: widget.type,
              value: item,
              onChanged: (val) {
                setState(() {
                  _newdData = _newdData
                      .asMap()
                      .map(
                          (key, value) => MapEntry(key, key == i ? val : value))
                      .values
                      .map<String>((e) => e)
                      .toList();
                });
              },
            ),
          ),
          InkWell(
            onTap: () {
              setState(() {
                _newdData.removeWhere((element) {
                  return element == item;
                });
              });
            },
            child: ThemedCircleAvatar(
              backgroundColor: Colors.red,
              child: ThemedIcons.clear(
                iconStyle: IconStyle(
                  color: Colors.white,
                ),
              ),
            ),
          )
        ],
      ),
    );
  }
}

class _EditTextFieldListItem extends StatelessWidget {
  final String value;
  final Function(String name) onChanged;
  final ListType type;
  const _EditTextFieldListItem({
    Key? key,
    required this.onChanged,
    required this.type,
    required this.value,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        sizedBoxW8,
        Center(
          child: SelectableText(
            value,
            style: const TextStyle(
              overflow: TextOverflow.fade,
            ),
          ),
        ),
        spacer,
        InkWell(
          onTap: () async {
            await sizedDialog(
              context: context,
              height: 250,
              width: 400,
              builder: (context) => Center(
                child: _EditTextFieldListItemContent(
                  onChanged: onChanged,
                  type: type,
                  value: value,
                ),
              ),
            );
          },
          child: ThemedCircleAvatar(
            child: ThemedIcons.edit(),
          ),
        )
      ],
    );
  }
}

class _EditTextFieldListItemContent extends StatefulWidget {
  final String value;
  final Function(String name) onChanged;
  final ListType type;

  const _EditTextFieldListItemContent({
    Key? key,
    required this.value,
    required this.onChanged,
    required this.type,
  }) : super(key: key);

  @override
  _EditTextFieldListItemContentState createState() =>
      _EditTextFieldListItemContentState();
}

class _EditTextFieldListItemContentState
    extends State<_EditTextFieldListItemContent> {
  TextEditingController? _controller;
  String _value = "";
  String _newData = "";

  @override
  void initState() {
    super.initState();
    final data = DataClassUtils.dynamicToString(widget.value);
    _value = data;
    _newData = data;
    _controller = TextEditingController(
      text: data,
    );

    _controller!.selection = TextSelection.fromPosition(
        TextPosition(offset: _controller!.text.length));
  }

  @override
  void dispose() {
    _controller!.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant _EditTextFieldListItemContent oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      final data = DataClassUtils.dynamicToString(widget.value);
      _value = data;
      _newData = data;
      _controller = TextEditingController(
        text: data,
      );
      _controller!.selection = TextSelection.fromPosition(
        TextPosition(
          offset: _controller!.text.length,
        ),
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: ThemedAppBar(
        navigateBackButton: true,
        actions: [
          ThemedIconButton(
            onPressed: () {
              setState(() {
                _controller = _controller = TextEditingController(
                  text: _value,
                );
                _newData = _value;
              });
              Navigator.of(context).pop();
            },
            icon: ThemedIcons.clear(
              iconStyle: IconStyle(
                color: Colors.redAccent,
              ),
            ),
          ),
          ThemedIconButton(
            onPressed: () {
              if (widget.type == ListType.email && !isEmail(_newData)) {
                ThemedToast.error(
                  context: context,
                  message: "Please enter a valid email",
                );
              } else {
                widget.onChanged(_newData);

                setState(() {
                  _controller = _controller = TextEditingController(
                    text: _newData,
                  );
                  _controller!.text = _newData;
                  _value = _newData;
                });
                Navigator.of(context).pop();
              }
            },
            icon: ThemedIcons.check(
              iconStyle: IconStyle(
                color: Colors.lightBlue,
              ),
            ),
          )
        ],
      ),
      body: Padding(
        padding: const EdgeInsets.all(8.0),
        child: TextField(
          decoration: const InputDecoration(
            label: Text('Edit line'),
          ),
          autofocus: false,
          onChanged: (val) {
            setState(() {
              _newData = val;
            });
          },
          controller: _controller,
        ),
      ),
    );
  }
}

class _AddTextFieldListItem extends StatelessWidget {
  final Function(String name) onChanged;
  final ListType type;
  const _AddTextFieldListItem({
    Key? key,
    required this.onChanged,
    required this.type,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ThemedIconButton(
      icon: ThemedIcons.add(),
      onPressed: () async {
        await sizedDialog(
          context: context,
          height: 250,
          width: 400,
          builder: (context) => Center(
            child: _AddTextFieldListItemContanet(
              onChanged: onChanged,
              type: type,
            ),
          ),
        );
      },
    );
  }
}

class _AddTextFieldListItemContanet extends StatefulWidget {
  final Function(String name) onChanged;
  final ListType type;

  const _AddTextFieldListItemContanet({
    Key? key,
    required this.onChanged,
    required this.type,
  }) : super(key: key);

  @override
  _AddTextFieldListItemContanetState createState() =>
      _AddTextFieldListItemContanetState();
}

class _AddTextFieldListItemContanetState
    extends State<_AddTextFieldListItemContanet> {
  TextEditingController? _controller;
  String _newLine = "";

  @override
  void initState() {
    super.initState();

    _controller = TextEditingController(
      text: '',
    );
    _controller!.selection = TextSelection.fromPosition(
        TextPosition(offset: _controller!.text.length));
  }

  @override
  void dispose() {
    _controller!.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant _AddTextFieldListItemContanet oldWidget) {
    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: ThemedAppBar(
        navigateBackButton: true,
        actions: [
          ThemedIconButton(
            onPressed: () {
              setState(() {
                _newLine = "";
                Navigator.of(context).pop();
              });
            },
            icon: ThemedIcons.clear(
              iconStyle: IconStyle(
                color: Colors.redAccent,
              ),
            ),
          ),
          ThemedIconButton(
            onPressed: () {
              String temp = _newLine;
              if (widget.type == ListType.email && !isEmail(temp)) {
                ThemedToast.error(
                  context: context,
                  message: "Please enter a valid email",
                );
              } else {
                setState(() {
                  _newLine = "";
                });
                widget.onChanged(temp);

                Navigator.of(context).pop();
              }
            },
            icon: ThemedIcons.check(
              iconStyle: IconStyle(
                color: Colors.lightBlue,
              ),
            ),
          )
        ],
      ),
      body: Padding(
        padding: const EdgeInsets.all(8.0),
        child: TextField(
          decoration: const InputDecoration(
            label: Text('Add line'),
          ),
          autofocus: false,
          onChanged: (val) {
            setState(() {
              _newLine = val;
            });
          },
          controller: _controller,
        ),
      ),
    );
  }
}
