// 🐦 Flutter imports:
import "package:flutter/material.dart";
import 'package:flutter/services.dart';

// 📦 Package imports:
import "package:collection/collection.dart";
import 'package:day_night_time_picker/day_night_time_picker.dart';
import 'package:day_night_time_picker/lib/constants.dart';
import 'package:verttice_logic/verttice_logic.dart';

// 🌎 Project imports:
import "package:verttice_ui/src/const/icons.dart";
import 'package:verttice_ui/src/const/box_sizes.dart';
import 'package:verttice_ui/src/widgets/border_label.dart';
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';

class TimeRangeFormFactory extends StatefulWidget {
  final Function(String name, dynamic value) onChanged;
  final String displayName;
  final String name;
  final dynamic value;
  final FormFieldState<dynamic>? state;
  final String? tip;

  const TimeRangeFormFactory({
    Key? key,
    required this.onChanged,
    required this.displayName,
    required this.name,
    required this.value,
    this.state,
    this.tip,
  }) : super(key: key);

  @override
  State<TimeRangeFormFactory> createState() => _TimeRangeFormFactoryState();
}

class _TimeRangeFormFactoryState extends State<TimeRangeFormFactory> {
  var _value = <String, int?>{};

  @override
  void initState() {
    super.initState();
    _value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value)
        .cast<String, int?>();
  }

  @override
  void didUpdateWidget(covariant TimeRangeFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    final value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value);
    if (!const MapEquality().equals(_value, value)) {
      _value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value)
          .cast<String, int?>();
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      tip: widget.tip,
      onCopied: () async {
        await Clipboard.setData(
          ClipboardData(
            text:
                "${_value["startTime"] != null && _value["startTime"] is int ? Dates.formatDataAndTime(milliseconds: _value["startTime"]!) : ""} - ${_value["endTime"] != null && _value["endTime"] is int ? Dates.formatDataAndTime(milliseconds: _value["endTime"]!) : ""}",
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Copied to clipboard",
        );
      },
      actions: [
        ThemedPhysicalModel(
          color: Colors.grey,
          elevation: 5,
          shape: BoxShape.circle,
          child: ThemedCircleAvatar(
            backgroundColor: Colors.red,
            child: InkWell(
              child: ThemedIcons.clear(
                iconStyle: IconStyle(
                  color: Colors.white,
                ),
              ),
              onTap: () {
                widget.onChanged(widget.name, null);
                widget.state?.didChange(null);
              },
            ),
          ),
        ),
        sizedBoxW2,
      ],
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          ThemedCleanTextButton(
            _value["startTime"] != null && _value["startTime"] is int
                ? Dates.formatDataAndTime(milliseconds: _value["startTime"]!)
                : "Start Date",
            onPressed: () async {
              DateTime? initialDate;
              if (_value['startTime'] != null && _value['startTime'] is int) {
                initialDate = DateTime.fromMillisecondsSinceEpoch(
                  _value['startTime']!,
                ).toLocal();
              }

              showDialog(
                context: context,
                builder: (context) => AlertDialog(
                  content: createInlinePicker(
                    value: TimeOfDay.fromDateTime(
                      initialDate ?? DateTime.now(),
                    ),
                    minuteInterval: MinuteInterval.FIVE,
                    onChange: (time) {
                      final currentDate =
                          DateTime.fromMillisecondsSinceEpoch(0);
                      int timeInDate = DateTime(
                        currentDate.year,
                        currentDate.month,
                        currentDate.day,
                        time.hour,
                        time.minute,
                      ).toUtc().millisecondsSinceEpoch;

                      widget.onChanged(widget.name, timeInDate);
                      widget.state?.didChange(timeInDate);
                    },
                  ),
                ),
              );
            },
            icon: ThemedIcons.calendar(),
          ),
          if (_value['endTime'] != null && _value['endTime'] is int)
            ThemedCleanTextButton(
              _value["endTime"] != null && _value["endTime"] is int
                  ? Dates.formatDataAndTime(milliseconds: _value["endTime"]!)
                  : "End Date",
              onPressed: () async {
                DateTime? initialDate;
                if (_value['endTime'] != null && _value['endTime'] is int) {
                  initialDate =
                      DateTime.fromMillisecondsSinceEpoch(_value['endTime']!)
                          .toLocal();
                }

                showDialog(
                  context: context,
                  builder: (context) => AlertDialog(
                    content: createInlinePicker(
                      value: TimeOfDay.fromDateTime(
                        initialDate ?? DateTime.now(),
                      ),
                      minuteInterval: MinuteInterval.FIVE,
                      onChange: (time) {
                        final currentDate =
                            DateTime.fromMillisecondsSinceEpoch(0);
                        int timeInDate = DateTime(
                          currentDate.year,
                          currentDate.month,
                          currentDate.day,
                          time.hour,
                          time.minute,
                        ).toUtc().millisecondsSinceEpoch;

                        widget.onChanged(widget.name, timeInDate);
                        widget.state?.didChange(timeInDate);
                      },
                    ),
                  ),
                );
              },
              icon: ThemedIcons.calendar(),
            ),
        ],
      ),
    );
  }
}
