// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:collection/collection.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class TablePagination extends StatelessWidget {
  final double height;
  final double width;
  final int pageLength;
  final int currentPage;
  final int itemsLength;
  final int maxPages;
  final void Function(int rowsPerPage) onRowsPerPageChanged;
  final void Function(int currentPage) onPageBack;
  final void Function(int currentPage) onPageForwared;
  final List<int> rowsPerPageOptions;
  const TablePagination({
    Key? key,
    required this.height,
    required this.width,
    required this.pageLength,
    required this.currentPage,
    required this.itemsLength,
    required this.maxPages,
    required this.onRowsPerPageChanged,
    required this.onPageBack,
    required this.onPageForwared,
    required this.rowsPerPageOptions,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: height,
      width: width,
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.end,
        mainAxisAlignment: MainAxisAlignment.end,
        children: [
          Stack(
            children: [
              const SizedBox(
                height: 25,
                width: 120,
              ),
              Positioned(
                top: 0,
                left: 0,
                child: Row(
                  children: const [
                    SelectableText("Rows per page: "),
                    SizedBox(width: 7),
                  ],
                ),
              ),
            ],
          ),
          SizedBox(
            height: 35,
            width: 70,
            child: PopupMenuButton(
              child: Center(
                child: Chip(
                  label: Text(
                    pageLength.toString(),
                  ),
                ),
              ),
              onSelected: (int index) {
                onRowsPerPageChanged(rowsPerPageOptions[index]);
              },
              itemBuilder: (BuildContext context) {
                return rowsPerPageOptions
                    .mapIndexed(
                      (int index, e) => PopupMenuItem(
                        value: index,
                        child: Text(e.toString()),
                      ),
                    )
                    .toList();
              },
            ),
          ),
          Stack(
            children: [
              const SizedBox(
                height: 25,
                width: 220,
              ),
              Positioned(
                top: 0,
                left: 10,
                child: Row(
                  children: [
                    sizedBoxW16,
                    SelectableText(
                      "${((currentPage * pageLength) + 1).toString()} - ${pageLength > itemsLength ? itemsLength.toString() : ((currentPage + 1) * pageLength).toString()} of  ${itemsLength.toString()} ",
                    ),
                    sizedBoxW16,
                    SelectableText(
                      "Page: ${(currentPage + 1).toString()} ",
                    ),
                    sizedBoxW16,
                  ],
                ),
              ),
            ],
          ),
          Stack(
            children: [
              const SizedBox(
                height: 30,
                width: 70,
              ),
              Positioned(
                  top: 0,
                  left: 0,
                  child: Row(children: [
                    InkWell(
                      onTap: () {
                        if (currentPage > 0) {
                          onPageBack(currentPage - 1);
                        }
                      },
                      child: ThemedIcons.arrowBack(
                        iconStyle: IconStyle(
                          color: currentPage > 0 ? Colors.black : Colors.grey,
                        ),
                      ),
                    ),
                    InkWell(
                      onTap: () {
                        if (currentPage < maxPages) {
                          onPageForwared(currentPage + 1);
                        }
                      },
                      child: ThemedIcons.arrowForwared(
                        iconStyle: IconStyle(
                          color: currentPage < maxPages
                              ? Colors.black
                              : Colors.grey,
                        ),
                      ),
                    ),
                    sizedBoxW16
                  ])),
            ],
          ),
        ],
      ),
    );
  }
}
