// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

Future<T?> fullPageDialog<T>({
  required BuildContext context,
  required Widget Function(BuildContext) builder,
}) async {
  if (ScreenSizeUtils.isMobile(context: context)) {
    return await Navigator.of(context).push(
      MaterialPageRoute<T>(
        builder: (BuildContext context) {
          return builder(context);
        },
        fullscreenDialog: true,
      ),
    );
  }
  return await showDialog<T>(
    context: context,
    builder: (BuildContext context) => AlertDialog(
      shape: const RoundedRectangleBorder(
        borderRadius: BorderRadius.all(
          Radius.circular(20.0),
        ),
      ),
      contentPadding: const EdgeInsets.all(10),
      content: SizedBox(
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.height,
        child: builder(context),
      ),
    ),
  );
}
