// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:provider/provider.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class LoadingDialog {
  LoadingDialog._privateConstructor();

  static final LoadingDialog instance = LoadingDialog._privateConstructor();

  BuildContext? _dialogContext;

  void openDialog({
    required BuildContext context,
    String? loadingMessage,
  }) async {
    showGeneralDialog(
      context: context,
      barrierDismissible: false,
      pageBuilder: (
        BuildContext context,
        Animation<double> _,
        Animation<double> __,
      ) {
        _dialogContext = context;
        return Consumer<VertticeUIState>(builder: (
          context,
          value,
          _,
        ) {
          return FittedBox(
            fit: BoxFit.scaleDown,
            child: SizedBox(
              height: 120,
              width: 200,
              child: Container(
                decoration: BoxDecoration(
                  color: ColorAdjust.lighten(
                    value.themedColors.accentColor,
                    percent: 75,
                  ),
                  borderRadius: const BorderRadius.all(
                    Radius.circular(25),
                  ),
                ),
                child: Center(
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      centeredLoader,
                      sizedBoxH32,
                      ThemedText(
                        loadingMessage ?? 'Loading...',
                        showThemed: true,
                        style: const TextStyle(
                          fontWeight: FontWeight.bold,
                        ),
                      ),
                    ],
                  ),
                ),
              ),
            ),
          );
        });
      },
    );
  }

  void closeDialog() {
    if (_dialogContext != null) {
      Navigator.pop(_dialogContext!);
      _dialogContext = null;
    }
  }
}
