// 🐦 Flutter imports:
import 'dart:math';

import "package:flutter/material.dart";

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

Future<T?> sizedDialog<T>({
  required BuildContext context,
  required double height,
  required double width,
  required Widget Function(BuildContext context) builder,
  ThemedAppBar Function(BuildContext context)? appBarBuilder,
  bool barrierDismissible = true,
  List<Widget>? actions,
}) async {
  return await showDialog<T>(
    context: context,
    useSafeArea: true,
    barrierDismissible: barrierDismissible,
    builder: (BuildContext context) {
      return AlertDialog(
        shape: const RoundedRectangleBorder(
          borderRadius: BorderRadius.all(
            Radius.circular(20.0),
          ),
        ),
        insetPadding: const EdgeInsets.all(15),
        contentPadding: const EdgeInsets.all(15),
        actions: actions,
        content: SizedBox(
          height: min(height, MediaQuery.of(context).size.height - 30),
          width: min(width, MediaQuery.of(context).size.width - 30),
          child: Scaffold(
            appBar: (appBarBuilder != null ? appBarBuilder(context) : null),
            body: builder(context),
          ),
        ),
      );
    },
  );
}
