// 🎯 Dart imports:
import 'dart:typed_data';

// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import 'package:verttice_ui/src/const/icons.dart';
import 'package:verttice_ui/src/utils/download.dart';
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';

class DownloadButton extends StatelessWidget {
  final String? url;
  final String? path;
  final String? asset;
  final Uint8List? data;
  final String? name;
  final VFileType fileType;

  const DownloadButton({
    Key? key,
    required this.fileType,
    this.url,
    this.path,
    this.asset,
    this.data,
    this.name,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ThemedIconButton(
      onPressed: () async {
        try {
          if (data == null && url == null && path == null && asset == null) {
            throw 'Cannot download file';
          }
          if (data != null) {
            await Download.memory(
              data: data!,
              fileType: fileType,
              name: name,
            );
          } else if (url != null) {
            await Download.network(
              url: url!,
              fileType: fileType,
              name: name,
            );
          } else if (asset != null) {
            await Download.asset(
              assetPath: asset!,
              fileType: fileType,
              name: name,
            );
          } else {
            await Download.storage(
              path: path!,
              fileType: fileType,
              name: name,
            );
          }
        } catch (e) {
          ThemedToast.error(
            context: context,
            message: 'Failed to download file',
          );
        }
      },
      icon: ThemedIcons.share(),
    );
  }
}
