// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import 'package:verttice_ui/src/const/icons.dart';
import 'images/image_viewer_full_page.dart';
import 'pdfs/pdf_viewer_full_page.dart';
import 'videos/video_viewer_full_page.dart';

class FullPageFileViewNetwork extends StatefulWidget {
  final FileModel file;
  final String? fileName;
  final bool? navigateBackButton;

  const FullPageFileViewNetwork({
    Key? key,
    required this.file,
    this.fileName,
    this.navigateBackButton,
  }) : super(key: key);

  @override
  State createState() => _FullPageFileViewNetworkState();
}

class _FullPageFileViewNetworkState extends State<FullPageFileViewNetwork> {
  late FileModel _file;
  @override
  void initState() {
    super.initState();
    _file = widget.file;
  }

  @override
  void didUpdateWidget(covariant FullPageFileViewNetwork oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_file != widget.file) {
      _file = widget.file;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Builder(
      builder: (BuildContext context) {
        if (_file.mimeType == null || _file.url == null) {
          return Center(
            child: ThemedIcons.unknown(
              iconStyle: IconStyle(
                color: Colors.amber[600],
                size: 45,
              ),
            ),
          );
        }
        if (_file.mimeType!.contains("image")) {
          return ImageViewerFullPage(
            url: _file.url,
            imageID: _file.url!,
            navigateBackButton: widget.navigateBackButton,
          );
        } else if (_file.mimeType!.contains("pdf")) {
          return PdfViewerFullPage(
            url: _file.url!,
            fileName: widget.fileName,
            navigateBackButton: widget.navigateBackButton,
          );
        } else if (_file.mimeType!.contains("video")) {
          return VideoViewerFullPage(
            url: _file.url!,
            navigateBackButton: widget.navigateBackButton,
          );
        }
        return Center(
          child: ThemedIcons.unknown(
            iconStyle: IconStyle(
              color: Colors.amber[600],
              size: 45,
            ),
          ),
        );
      },
    );
  }
}
