// 🐦 Flutter imports:
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

// 📦 Package imports:
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class FileItem extends StatelessWidget {
  final FileModel file;
  final bool fromStorage;
  final bool fromMemory;
  final double? height;
  final double? width;
  final Function(FileModel imageToDelete)? deleteFileCallback;

  const FileItem({
    Key? key,
    required this.file,
    this.fromStorage = false,
    this.fromMemory = false,
    this.height,
    this.width,
    this.deleteFileCallback,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    const double sideButtonsWidth = 40;
    final double? fileWidth = width != null ? width! - sideButtonsWidth : null;
    return Row(
      crossAxisAlignment: CrossAxisAlignment.center,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        if (deleteFileCallback != null || file.url != null)
          SizedBox(
            width: sideButtonsWidth,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.center,
              mainAxisAlignment: MainAxisAlignment.end,
              children: [
                if (file.url != null)
                  Tooltip(
                    message: "Copy file's link",
                    child: InkWell(
                      child: const SizedBox(
                        height: 30,
                        width: 40,
                        child: Icon(
                          Icons.copy,
                          color: Colors.lightBlue,
                          size: 17,
                        ),
                      ),
                      onTap: () async {
                        await Clipboard.setData(
                          ClipboardData(
                            text: file.url,
                          ),
                        );
                        ThemedToast.info(
                          context: context,
                          message: "File LINK copied to clipboard",
                        );
                      },
                    ),
                  ),
                if (deleteFileCallback != null)
                  Tooltip(
                    message: 'Delete file',
                    child: InkWell(
                      child: const SizedBox(
                        height: 30,
                        width: 40,
                        child: Icon(
                          Icons.delete_outlined,
                          color: Colors.red,
                          size: 19,
                        ),
                      ),
                      onTap: () {
                        showDialog(
                          context: context,
                          builder: (BuildContext context) {
                            return AlertDialog(
                              title: const SelectableText("Delete file"),
                              content: const SelectableText(
                                "Are you sure you want to delete this file?",
                              ),
                              actions: [
                                TextButton(
                                  child: const Text("OK"),
                                  onPressed: () {
                                    if (deleteFileCallback != null) {
                                      deleteFileCallback!(file);
                                    }
                                    Navigator.of(context).pop();
                                  },
                                ),
                                TextButton(
                                  child: const Text("Cancel"),
                                  onPressed: () {
                                    Navigator.of(context).pop();
                                  },
                                ),
                              ],
                            );
                          },
                        );
                      },
                    ),
                  ),
              ],
            ),
          ),
        Builder(
          builder: (BuildContext context) {
            if (file.mimeType == null) {
              return SizedBox(
                height: height,
                width: fileWidth,
                child: Center(
                  child: ThemedIcons.unknown(
                    iconStyle: IconStyle(
                      color: Colors.red,
                      size: 45,
                    ),
                  ),
                ),
              );
            }
            if (fromStorage && file.path != null) {
              if (file.mimeType!.contains("image")) {
                return ImageViewerStorage(
                  height: height,
                  width: fileWidth,
                  path: file.path!,
                );
              } else if (file.mimeType!.contains("pdf")) {
                return PdfViewerStorage(
                  height: height,
                  width: fileWidth,
                  path: file.path!,
                );
              } else if (file.mimeType!.contains("video")) {
                return VideoViewerStorage(
                  height: height,
                  width: fileWidth,
                  path: file.path!,
                );
              }
            } else if (file.url != null || file.file != null) {
              if (file.mimeType!.contains("image")) {
                return ImageViewer(
                  height: height,
                  width: fileWidth,
                  url: fromMemory ? null : file.url,
                  data: fromMemory ? file.file : null,
                );
              } else if (file.mimeType!.contains("pdf")) {
                return PdfViewer(
                  height: height,
                  width: fileWidth,
                  url: fromMemory ? null : file.url,
                  data: fromMemory ? file.file : null,
                );
              } else if (file.mimeType!.contains("video")) {
                return VideoViewer(
                  height: height,
                  width: fileWidth,
                  url: fromMemory ? null : file.url,
                  data: fromMemory ? file.file : null,
                );
              }
            }

            return SizedBox(
              height: height,
              width: fileWidth,
              child: Center(
                child: ThemedIcons.unknown(
                  iconStyle: IconStyle(
                    color: Colors.red,
                    size: 45,
                  ),
                ),
              ),
            );
          },
        ),
      ],
    );
  }
}
