// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import 'file_item.dart';

class FileListView extends StatefulWidget {
  final List<FileModel> files;
  final bool fromStorage;
  final bool fromMemory;
  final bool showAsGrid;
  final Axis scrollDirection;
  final double? itemHeight;
  final double? itemWidth;
  final Function(FileModel imageToDelete)? deleteFileCallback;
  const FileListView({
    Key? key,
    required this.files,
    this.fromStorage = false,
    this.fromMemory = false,
    this.deleteFileCallback,
    this.showAsGrid = false,
    this.scrollDirection = Axis.vertical,
    this.itemHeight,
    this.itemWidth,
  }) : super(key: key);

  @override
  State createState() => _FileListViewState();
}

class _FileListViewState extends State<FileListView> {
  List<FileModel> _files = [];
  @override
  void initState() {
    super.initState();
    _files = widget.files;
  }

  @override
  void didUpdateWidget(covariant FileListView oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_files != widget.files) {
      _files = widget.files;
    }
  }

  @override
  Widget build(BuildContext context) {
    if (widget.showAsGrid) {
      return Center(
        child: SingleChildScrollView(
          scrollDirection: widget.scrollDirection,
          child: Wrap(
            children: _files
                .map(
                  (e) => Padding(
                    padding: const EdgeInsets.all(18.0),
                    child: SizedBox(
                      width: widget.itemWidth ?? 330,
                      height: widget.itemHeight ?? 250,
                      child: FileItem(
                        width: widget.itemWidth ?? 330,
                        height: widget.itemHeight ?? 250,
                        fromMemory: widget.fromMemory,
                        fromStorage: widget.fromStorage,
                        file: e,
                        deleteFileCallback: widget.deleteFileCallback,
                      ),
                    ),
                  ),
                )
                .toList(),
          ),
        ),
      );
    }
    return ListView(
      scrollDirection: widget.scrollDirection,
      children: _files
          .map(
            (e) => SizedBox(
              width: widget.itemWidth,
              height: widget.itemHeight,
              child: Padding(
                padding: const EdgeInsets.all(6.0),
                child: FileItem(
                  height:
                      widget.itemHeight != null ? widget.itemHeight! - 6 : null,
                  width:
                      widget.itemWidth != null ? widget.itemWidth! - 6 : null,
                  fromStorage: widget.fromStorage,
                  fromMemory: widget.fromMemory,
                  file: e,
                  deleteFileCallback: widget.deleteFileCallback,
                ),
              ),
            ),
          )
          .toList(),
    );
  }
}
