// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import "package:verttice_ui/src/const/box_sizes.dart";
import 'package:verttice_ui/src/widgets/files/file_list_view.dart';
import 'package:verttice_ui/src/widgets/files/upload_file.dart';

class FileUploadAndView extends StatelessWidget {
  final List<FileModel>? files;
  final Function(FileModel)? deleteFileCallback;
  final Future<void> Function({
    required List<FileModel> files,
  }) filesDataCallback;
  final String path;
  final String? buttonText;
  final bool multi;
  final bool viewFromStorage;
  final VoidCallback? onDoneUpload;

  const FileUploadAndView({
    Key? key,
    required this.path,
    required this.filesDataCallback,
    required this.viewFromStorage,
    this.files,
    this.deleteFileCallback,
    this.buttonText,
    this.multi = true,
    this.onDoneUpload,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      child: Column(
        children: [
          UploadFile(
            multi: multi,
            buttonText: buttonText,
            filesDataCallback: filesDataCallback,
            path: path,
            onDoneUpload: onDoneUpload,
          ),
          sizedBoxH16,
          if (files != null && files!.isNotEmpty)
            Expanded(
              child: Center(
                child: FileListView(
                  showAsGrid: true,
                  fromStorage: viewFromStorage,
                  itemHeight: 330,
                  deleteFileCallback: deleteFileCallback,
                  files: files ?? [],
                ),
              ),
            ),
        ],
      ),
    );
  }
}
