// 🎯 Dart imports:
import "dart:typed_data";

// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import "package:photo_view/photo_view.dart";
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import 'package:verttice_ui/src/const/box_sizes.dart';
import 'package:verttice_ui/src/const/icons.dart';
import 'package:verttice_ui/src/widgets/download_button.dart';
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';

class ImageViewerFullPage extends StatefulWidget {
  final String? url;
  final String? asset;
  final Uint8List? data;
  final String imageID;
  final bool? navigateBackButton;

  const ImageViewerFullPage({
    Key? key,
    this.url,
    this.asset,
    this.data,
    this.navigateBackButton,
    required this.imageID,
  }) : super(key: key);

  @override
  ImageViewerFullPageState createState() => ImageViewerFullPageState();
}

class ImageViewerFullPageState extends State<ImageViewerFullPage> {
  int _quarterTurns = 0;
  @override
  Widget build(BuildContext context) {
    ImageProvider<Object> provider;
    late DownloadButton downloadWidget;
    if (widget.url != null) {
      provider = NetworkImage(widget.url!);
      downloadWidget = DownloadButton(
        url: widget.url!,
        fileType: VFileType.png,
      );
    } else if (widget.data != null) {
      provider = MemoryImage(widget.data!);
      downloadWidget = DownloadButton(
        data: widget.data!,
        fileType: VFileType.png,
      );
    } else {
      provider = AssetImage(widget.asset!);
      downloadWidget = DownloadButton(
        asset: widget.asset!,
        fileType: VFileType.png,
      );
    }
    return Scaffold(
      appBar: ThemedAppBar(
        navigateBackButton: widget.navigateBackButton ?? true,
        actions: [
          sizedBoxW8,
          ThemedCircleAvatar(
            child: downloadWidget,
          ),
          sizedBoxW8,
          ThemedCircleAvatar(
            child: ThemedIconButton(
              onPressed: () {
                setState(() {
                  _quarterTurns =
                      _quarterTurns + 1 == 4 ? 0 : _quarterTurns + 1;
                });
              },
              icon: ThemedIcons.refresh(),
            ),
          ),
          sizedBoxW8,
        ],
      ),
      body: Container(
        color: Colors.blueGrey[700]!.withOpacity(0.5),
        child: RotatedBox(
          quarterTurns: _quarterTurns,
          child: PhotoView(
            backgroundDecoration: const BoxDecoration(
              color: Colors.transparent,
            ),
            imageProvider: provider,
            heroAttributes: PhotoViewHeroAttributes(tag: widget.imageID),
            initialScale: PhotoViewComputedScale.contained,
          ),
        ),
      ),
    );
  }
}
