// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 🌎 Project imports:
import 'package:verttice_ui/src/config/config.dart';
import 'package:verttice_ui/src/const/icons.dart';
import 'package:verttice_ui/src/widgets/snap_builder.dart';
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';
import 'image_viewer.dart';

class ImageViewerStorage extends StatefulWidget {
  final String path;
  final double? height;
  final double? width;
  const ImageViewerStorage({
    Key? key,
    required this.path,
    this.height,
    this.width,
  }) : super(key: key);
  @override
  State createState() => _ImageViewerStorageState();
}

class _ImageViewerStorageState extends State<ImageViewerStorage> {
  late String _path;
  late Future<String?> _fetchData;
  @override
  void initState() {
    super.initState();
    _path = widget.path;
    _fetchData = VertticeUIConfig.instance.getStorageDownloadURL(path: _path);
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant ImageViewerStorage oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_path != widget.path) {
      _path = widget.path;
      _fetchData = VertticeUIConfig.instance.getStorageDownloadURL(path: _path);
    }
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<String?>(
        future: _fetchData,
        builder: (BuildContext context, AsyncSnapshot<String?> snapdata) {
          return SnapBuilder<String?>(
              snapshot: snapdata,
              height: widget.height,
              width: widget.width,
              errorBuilder: (dynamic error) => SizedBox(
                    height: widget.height,
                    width: widget.width,
                    child: Center(
                      child: ThemedCircleAvatar(
                        backgroundColor: Colors.red,
                        child: ThemedIcons.unknown(
                          iconStyle: IconStyle(
                            color: Colors.white,
                            size: 35,
                          ),
                        ),
                      ),
                    ),
                  ),
              loadingBuilder: () => const ThemedProgress(),
              successBuilder: (String? data) {
                return ImageViewer(
                  url: data ?? "",
                  height: widget.height,
                );
              });
        });
  }
}
