// 🎯 Dart imports:
import 'dart:typed_data';

// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:collection/collection.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class PdfViewer extends StatefulWidget {
  final String? url;
  final Uint8List? data;
  final double? height;
  final double? width;
  final String? displayName;
  final VoidCallback? onOpened;
  PdfViewer({
    Key? key,
    this.url,
    this.data,
    this.height,
    this.width,
    this.displayName,
    this.onOpened,
  }) : super(key: key) {
    assert(
      url != null || data != null,
      "Must provide url or data",
    );
  }
  @override
  State<PdfViewer> createState() => _PdfViewerState();
}

class _PdfViewerState extends State<PdfViewer> {
  String? _url;
  Uint8List? _data;

  @override
  void initState() {
    _url = widget.url;
    _data = widget.data;
    super.initState();
  }

  @override
  void didUpdateWidget(covariant PdfViewer oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_url != widget.url ||
        !const DeepCollectionEquality().equals(_data, widget.data)) {
      _url = widget.url;
      _data = widget.data;
    }
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: widget.height,
      width: widget.width,
      child: InkWell(
        onTap: () async {
          if (widget.onOpened != null) {
            Future.delayed(const Duration(seconds: 5)).then((value) {
              widget.onOpened!();
            });
          }
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => PdfViewerFullPage(
                url: _url,
                fileName: widget.displayName != null
                    ? widget.displayName!.replaceAll(' ', '_')
                    : null,
                data: _data,
              ),
            ),
          );
        },
        child: Container(
          decoration: const BoxDecoration(
            color: Color.fromARGB(255, 255, 254, 228),
            borderRadius: BorderRadius.all(
              Radius.circular(10.0),
            ),
          ),
          child: Center(
            child: ThemedIcons.pdfFile(
              iconStyle: IconStyle(
                size: 35,
                color: Colors.red,
              ),
            ),
          ),
        ),
      ),
    );
  }
}
