// 🎯 Dart imports:
import 'dart:typed_data';

// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:pdf/pdf.dart';
import 'package:printing/printing.dart';
import 'package:verttice_logic/verttice_logic.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class PdfViewerFullPage extends StatelessWidget {
  final String? url;
  final Uint8List? data;
  final String? fileName;
  final bool? navigateBackButton;

  PdfViewerFullPage({
    Key? key,
    this.url,
    this.data,
    this.fileName,
    this.navigateBackButton,
  }) : super(key: key) {
    assert(
      url != null || data != null,
      "Must provide url or data",
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: navigateBackButton != null && !navigateBackButton!
          ? null
          : const ThemedAppBar(
              navigateBackButton: true,
            ),
      body: PdfPreview(
        canChangeOrientation: false,
        pdfFileName: fileName ??
            Dates.formatDataAndTime(
              milliseconds: DateTime.now().millisecondsSinceEpoch,
            ),
        build: (PdfPageFormat pdfPageFormat) async {
          if (data != null) {
            return data!;
          }
          final dataRes = await HttpRetry().get(
            url!,
            responseType: HttpRetryResponseType.bytes,
          );

          return dataRes.data;
        },
      ),
    );
  }
}
