// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class PdfViewerStorage extends StatefulWidget {
  final String path;
  final double? height;
  final double? width;
  final String? displayName;
  final VoidCallback? onOpened;

  const PdfViewerStorage({
    Key? key,
    required this.path,
    this.height,
    this.width,
    this.displayName,
    this.onOpened,
  }) : super(key: key);
  @override
  State createState() => _PdfViewerStorageState();
}

class _PdfViewerStorageState extends State<PdfViewerStorage> {
  late String _path;
  late Future<String?> _fetchData;
  @override
  void initState() {
    super.initState();
    _path = widget.path;
    _fetchData = VertticeUIConfig.instance.getStorageDownloadURL(path: _path);
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant PdfViewerStorage oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_path != widget.path) {
      _path = widget.path;
      _fetchData = VertticeUIConfig.instance.getStorageDownloadURL(path: _path);
    }
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<String?>(
      future: _fetchData,
      builder: (BuildContext context, AsyncSnapshot<String?> snapdata) {
        return SnapBuilder<String?>(
          snapshot: snapdata,
          height: widget.height,
          width: widget.width,
          errorBuilder: (dynamic error) => SizedBox(
            height: widget.height,
            width: widget.width,
            child: Center(
              child: ThemedCircleAvatar(
                backgroundColor: Colors.red,
                child: ThemedIcons.unknown(
                  iconStyle: IconStyle(
                    color: Colors.white,
                    size: 35,
                  ),
                ),
              ),
            ),
          ),
          loadingBuilder: () => const ThemedProgress(),
          successBuilder: (String? url) {
            return PdfViewer(
              onOpened: widget.onOpened,
              url: url ?? "",
              displayName: widget.displayName,
              height: widget.height,
              width: widget.width,
            );
          },
        );
      },
    );
  }
}
