// 🎯 Dart imports:
import 'dart:typed_data';

// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:collection/collection.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class VideoViewer extends StatefulWidget {
  final String? url;
  final Uint8List? data;
  final double? height;
  final double? width;
  VideoViewer({
    Key? key,
    this.url,
    this.data,
    this.height,
    this.width,
  }) : super(key: key) {
    assert(
      url != null || data != null,
      "Must provide url or data",
    );
  }
  @override
  State createState() => _VideoViewerState();
}

class _VideoViewerState extends State<VideoViewer> {
  String? _url;
  Uint8List? _data;

  @override
  void initState() {
    _url = widget.url;
    _data = widget.data;
    super.initState();
  }

  @override
  void didUpdateWidget(covariant VideoViewer oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_url != widget.url ||
        !const DeepCollectionEquality().equals(_data, widget.data)) {
      _url = widget.url;
      _data = widget.data;
    }
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: widget.height,
      width: widget.width,
      child: InkWell(
        onTap: () {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => VideoViewerFullPage(
                url: _url,
                data: _data,
              ),
            ),
          );
        },
        child: Container(
          decoration: const BoxDecoration(
            color: Color.fromARGB(255, 253, 234, 226),
            borderRadius: BorderRadius.all(
              Radius.circular(10.0),
            ),
          ),
          child: Center(
            child: ThemedIcons.video(
              iconStyle: IconStyle(
                size: 35,
                color: Colors.red,
              ),
            ),
          ),
        ),
      ),
    );
  }
}
