import 'package:flutter/material.dart';

import 'snap_builder.dart';

class FutureSnapBuilder<T> extends StatefulWidget {
  final Future<T> future;
  final T? initialData;
  final Widget Function(dynamic error)? errorBuilder;
  final Widget Function(T data) successBuilder;
  final Widget Function()? loadingBuilder;
  final Widget Function()? nullBuilder;
  final bool? isDataLoaded;
  final double? width;
  final double? height;

  const FutureSnapBuilder({
    Key? key,
    required this.future,
    required this.successBuilder,
    this.initialData,
    this.errorBuilder,
    this.nullBuilder,
    this.loadingBuilder,
    this.isDataLoaded,
    this.width,
    this.height,
  }) : super(key: key);

  @override
  State<FutureSnapBuilder<T>> createState() => _FutureSnapBuilderState();
}

class _FutureSnapBuilderState<T> extends State<FutureSnapBuilder<T>> {
  bool isDataLoadedIndicate = false;
  @override
  Widget build(BuildContext context) {
    return FutureBuilder<T>(
      future: widget.future,
      initialData: widget.initialData,
      builder: (
        BuildContext context,
        AsyncSnapshot<T> snapshot,
      ) {
        return SnapBuilder<T>(
          loaded: widget.isDataLoaded ?? isDataLoadedIndicate,
          snapshot: snapshot,
          successBuilder: (T data) {
            isDataLoadedIndicate = true;
            return widget.successBuilder(data);
          },
          errorBuilder: widget.errorBuilder,
          height: widget.height,
          width: widget.width,
          loadingBuilder: widget.loadingBuilder,
          nullBuilder: widget.nullBuilder,
        );
      },
    );
  }
}
