// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import "package:verttice_ui/src/const/box_sizes.dart";
import 'package:verttice_ui/src/widgets/drawing/painter.dart';
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';
import 'files/files_index.dart';

class HandSignature extends StatefulWidget {
  final Function(FileModel signature) signatureFileCallback;
  final String? currentSignaturePath;
  final String signaturePath;
  final String signatureText;

  const HandSignature({
    Key? key,
    required this.signatureFileCallback,
    required this.currentSignaturePath,
    required this.signaturePath,
    this.signatureText = "Client Signature",
  }) : super(key: key);

  @override
  State<HandSignature> createState() => _HandSignatureState();
}

class _HandSignatureState extends State<HandSignature> {
  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        if (widget.currentSignaturePath != null)
          Center(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.center,
              children: [
                Padding(
                  padding: const EdgeInsets.all(10),
                  child: SelectableText(
                    widget.signatureText,
                    style: const TextStyle(
                      fontSize: 17,
                    ),
                  ),
                ),
                Padding(
                    padding: const EdgeInsets.fromLTRB(5, 0, 5, 0),
                    child: SizedBox(
                      height: 150,
                      child: ImageViewerStorage(
                        path: widget.currentSignaturePath ?? "",
                      ),
                    )),
                sizedBoxH4,
              ],
            ),
          ),
        Center(
          child: ThemedTextButton(
            'Hand Siganture',
            onPressed: () {
              Navigator.push(
                context,
                MaterialPageRoute(
                  builder: (context) => Painter(
                    buttonText: "Update ${widget.signatureText}",
                    imageDataCallback: widget.signatureFileCallback,
                    path: widget.signaturePath,
                  ),
                ),
              );
            },
          ),
        ),
      ],
    );
  }
}
