// 🐦 Flutter imports:
import "package:flutter/material.dart";

class InteractiveViewerTap extends StatefulWidget {
  final Widget child;
  const InteractiveViewerTap({
    Key? key,
    required this.child,
  }) : super(key: key);

  @override
  State<InteractiveViewerTap> createState() => _InteractiveViewerTapState();
}

class _InteractiveViewerTapState extends State<InteractiveViewerTap> {
  final _transformationController = TransformationController();
  TapDownDetails? _doubleTapDetails;

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onDoubleTapDown: _handleDoubleTapDown,
      onDoubleTap: _handleDoubleTap,
      child: Center(
        child: InteractiveViewer(
            transformationController: _transformationController,
            clipBehavior: Clip.antiAlias,
            child: widget.child),
      ),
    );
  }

  void _handleDoubleTapDown(TapDownDetails details) {
    _doubleTapDetails = details;
  }

  void _handleDoubleTap() {
    if (_transformationController.value != Matrix4.identity()) {
      _transformationController.value = Matrix4.identity();
    } else {
      final position = _doubleTapDetails!.localPosition;
      // For a 3x zoom
      _transformationController.value = Matrix4.identity()
        ..translate(
          -position.dx * 2,
          -position.dy * 2,
        )
        ..scale(3.0);
      // Fox a 2x zoom
      // ..translate(-position.dx, -position.dy)
      // ..scale(2.0);
    }
  }
}
