// 🐦 Flutter imports:
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class Maps extends StatelessWidget {
  final double lat;
  final double lng;
  final String? placeName;
  final double mapHeight;
  final bool showCopyAddressButton;
  const Maps({
    Key? key,
    required this.lat,
    required this.lng,
    required this.mapHeight,
    this.placeName,
    this.showCopyAddressButton = true,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(builder: (context, constraines) {
      return SizedBox(
          height: mapHeight,
          width: constraines.maxWidth,
          child: ThemedTabBarContent(
            items: [
              ThemedMenuItem(
                  title: 'Map',
                  icon: Icons.map,
                  child: GoogleMaps(
                    lat: lat,
                    lng: lng,
                    mapHeight: mapHeight - 110,
                    mapWidth: constraines.maxWidth,
                    placeName: placeName,
                  )),
              ThemedMenuItem(
                title: 'Street view',
                icon: Icons.streetview,
                child: SizedBox(
                  height: mapHeight - 110,
                  width: constraines.maxWidth,
                  child: GoogleStreetView(
                    lat: lat,
                    lng: lng,
                  ),
                ),
              ),
            ],
            navigateBackButton: false,
            title: placeName ?? '',
            titleTextStyle: const TextStyle(fontSize: 11),
            actions: [
              if (showCopyAddressButton && placeName != null)
                ThemedIconButton(
                  onPressed: () async {
                    await Clipboard.setData(
                      ClipboardData(
                        text: placeName,
                      ),
                    );
                    ThemedToast.info(
                      context: context,
                      message: "Address copied to clipborad",
                    );
                  },
                  icon: ThemedIcons.copy(),
                )
            ],
          ));
    });
  }
}
