// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:provider/provider.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

/// if screen id mobile is stay the same otherwise it change to desktop layout
class MobileToDesktopPage extends StatefulWidget {
  final Widget mobilePage;
  final bool contentOnTheLeft;
  final Image imageForEmptySide;
  const MobileToDesktopPage({
    Key? key,
    required this.mobilePage,
    required this.imageForEmptySide,
    this.contentOnTheLeft = true,
  }) : super(key: key);

  @override
  State<MobileToDesktopPage> createState() => _MobileToDesktopPageState();
}

class _MobileToDesktopPageState extends State<MobileToDesktopPage> {
  @override
  Widget build(BuildContext context) {
    final Widget mobilePage = widget.mobilePage;
    final bool contentOnTheLeft = widget.contentOnTheLeft;
    return LayoutBuilder(
      builder: (context, constraints) {
        return ScreenHandler(
          keepStateAlive: false,
          mobileWidget: mobilePage,
          desktopWidget: Row(
            children: [
              if (contentOnTheLeft)
                SizedBox(
                  width: constraints.maxWidth * 0.5,
                  child: mobilePage,
                ),
              Consumer<VertticeUIState>(
                builder: (context, value, _) {
                  return Container(
                    width: constraints.maxWidth * 0.5,
                    color: value.isDarkTheme ? Colors.grey[850] : Colors.white,
                    child: Center(
                      child: Column(
                        mainAxisAlignment: MainAxisAlignment.center,
                        children: [
                          sizedBoxH64,
                          SizedBox(
                            width: (constraints.maxWidth * 0.5) * 0.8,
                            height: 300,
                            child: ThemedHoverEffect(
                              builder: (context, hovering) {
                                return Container(
                                  color: value.isDarkTheme
                                      ? Colors.grey[850]
                                      : Colors.white,
                                  child: Center(
                                    child: widget.imageForEmptySide,
                                  ),
                                );
                              },
                              depth: 10,
                              depthColor: value.themedColors.accentColor,
                              shadow: BoxShadow(
                                color:
                                    value.themedColors.accentColor.withOpacity(
                                  0.8,
                                ),
                                blurRadius: 30,
                                spreadRadius: -20,
                                offset: const Offset(0, 40),
                              ),
                            ),
                          ),
                          sizedBoxH64,
                          const Text(
                            'Click And Hover Me!',
                            style: TextStyle(
                                fontSize: 15,
                                fontWeight: FontWeight.w700,
                                color: Colors.black,
                                letterSpacing: 2),
                          ),
                        ],
                      ),
                    ),
                  );
                },
              ),
              if (!contentOnTheLeft)
                SizedBox(
                  width: constraints.maxWidth * 0.5,
                  child: mobilePage,
                ),
            ],
          ),
        );
      },
    );
  }
}
