// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class SaveDataHandler extends StatefulWidget {
  final Future<dynamic> Function() future;
  final bool navigateBackOnSaved;
  final VoidCallback? onDone;
  final Icon? icon;
  final String iconTooltip;
  final String successMessage;
  final String failedMessage;
  final List<FieldMustExist>? fieldsMustExistOnSubmit;
  final String? title;
  final bool showToast;
  final bool failHandler;
  final Duration waitBeforeEndMessage;

  const SaveDataHandler({
    Key? key,
    required this.future,
    this.navigateBackOnSaved = false,
    this.onDone,
    this.icon,
    this.iconTooltip = "Save changes",
    this.successMessage = 'Data saved successfully',
    this.failedMessage = 'Failed to save data',
    this.fieldsMustExistOnSubmit,
    this.showToast = true,
    this.title,
    this.failHandler = true,
    this.waitBeforeEndMessage = const Duration(milliseconds: 100),
  }) : super(key: key);

  @override
  State<SaveDataHandler> createState() => _SaveDataHandlerState();
}

class _SaveDataHandlerState extends State<SaveDataHandler> {
  bool _isSavingForm = false;

  Future<void> _saveData() async {
    if (_isSavingForm) {
      return;
    }
    _isSavingForm = true;

    setState(() {});
    bool updateSuccess = false;
    try {
      await widget.future();
      if (widget.onDone != null) {
        widget.onDone!.call();
      }
      updateSuccess = true;
      if (widget.showToast) {
        await Future.delayed(widget.waitBeforeEndMessage);
        ThemedToast.success(
          context: context,
          message: widget.successMessage,
        );
      }
    } catch (e) {
      debugPrint(e.toString());
      if (widget.showToast) {
        await Future.delayed(widget.waitBeforeEndMessage);

        ThemedToast.error(
          context: context,
          message: widget.failedMessage,
        );
      }
      if (!widget.failHandler) {
        rethrow;
      }
    } finally {
      setState(() {
        _isSavingForm = false;
      });
    }
    if (widget.navigateBackOnSaved && updateSuccess) {
      if (!mounted) return;

      Navigator.pop(context);
    }
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: 47,
      width: 47,
      child: Center(
        child: AnimatedSwitcher(
          duration: const Duration(milliseconds: 500),
          transitionBuilder: (Widget child, Animation<double> animation) {
            return ScaleTransition(scale: animation, child: child);
          },
          child: _isSavingForm
              ? ThemedProgress(
                  key: ValueKey<bool>(_isSavingForm),
                )
              : Tooltip(
                  key: ValueKey<bool>(!_isSavingForm),
                  message: widget.iconTooltip,
                  child: ThemedIconButton(
                    isOffMode: _isSavingForm,
                    onPressed: _isSavingForm ? () {} : _saveData,
                    icon: widget.icon ?? ThemedIcons.check(),
                  ),
                ),
        ),
      ),
    );
  }
}
