// 🐦 Flutter imports:
import "package:flutter/material.dart";
import 'package:flutter/foundation.dart';

// 🌎 Project imports:
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';

class SnapBuilder<T> extends StatelessWidget {
  final AsyncSnapshot<T> snapshot;
  final Widget Function(dynamic error)? errorBuilder;
  final Widget Function(T data) successBuilder;
  final Widget Function()? loadingBuilder;
  final Widget Function()? nullBuilder;
  final bool loaded;
  final double? width;
  final double? height;
  const SnapBuilder({
    Key? key,
    required this.snapshot,
    required this.successBuilder,
    this.errorBuilder,
    this.nullBuilder,
    this.loaded = false,
    this.loadingBuilder,
    this.width,
    this.height,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    if (loaded && snapshot.data != null) {
      return successBuilder(snapshot.data as T);
    } else if (snapshot.connectionState == ConnectionState.waiting) {
      if (loadingBuilder != null) {
        return loadingBuilder!();
      } else {
        return const Center(
          child: FittedBox(
            fit: BoxFit.contain,
            child: ThemedProgress(),
          ),
        );
      }
    } else if (snapshot.hasData) {
      if (snapshot.data == null) {
        if (nullBuilder != null) {
          return nullBuilder!();
        }
        return ThemedMessage(
          message: "Nothing to show...",
          height: height,
          width: width,
        );
      } else {
        return successBuilder(snapshot.data as T);
      }
    } else if (snapshot.hasError) {
      if (kDebugMode) {
        print(snapshot.error);
      }
      if (errorBuilder != null) {
        return errorBuilder!(snapshot.error);
      }
      return ThemedMessage(
        message: "Failed to load data...",
        height: height,
        width: width,
      );
    }

    return ThemedMessage(
      message: "Nothing to show...",
      height: height,
      width: width,
    );
  }
}
