import 'package:flutter/material.dart';

import 'snap_builder.dart';

class StreamSnapBuilder<T> extends StatefulWidget {
  final Stream<T> stream;
  final T? initialData;
  final Widget Function(dynamic error)? errorBuilder;
  final Widget Function(T data) successBuilder;
  final Widget Function()? loadingBuilder;
  final Widget Function()? nullBuilder;
  final double? width;
  final double? height;

  const StreamSnapBuilder({
    Key? key,
    required this.stream,
    required this.successBuilder,
    this.initialData,
    this.errorBuilder,
    this.nullBuilder,
    this.loadingBuilder,
    this.width,
    this.height,
  }) : super(key: key);

  @override
  State<StreamSnapBuilder<T>> createState() => _StreamSnapBuilderState<T>();
}

class _StreamSnapBuilderState<T> extends State<StreamSnapBuilder<T>> {
  bool isDataLoaded = false;

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<T>(
      stream: widget.stream,
      initialData: widget.initialData,
      builder: (
        BuildContext context,
        AsyncSnapshot<T> snapshot,
      ) {
        isDataLoaded = true;
        return SnapBuilder<T>(
          loaded: isDataLoaded,
          snapshot: snapshot,
          successBuilder: widget.successBuilder,
          errorBuilder: widget.errorBuilder,
          height: widget.height,
          width: widget.width,
          loadingBuilder: widget.loadingBuilder,
          nullBuilder: widget.nullBuilder,
        );
      },
    );
  }
}
