// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:fluent_ui/fluent_ui.dart' as fluent;

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class ThemedAppBar extends StatelessWidget with PreferredSizeWidget {
  final String? title;
  final List<Widget>? actions;
  final Color? backgroundColor;
  final bool navigateBackButton;
  final Widget? leading;
  final double appBarHeight;
  final TextStyle? titleTextStyle;

  const ThemedAppBar({
    Key? key,
    required this.navigateBackButton,
    this.title,
    this.titleTextStyle,
    this.actions,
    this.backgroundColor,
    this.leading,
    this.appBarHeight = 50,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: appBarHeight,
      child: fluent.NavigationView(
        appBar: fluent.NavigationAppBar(
          height: appBarHeight,
          backgroundColor: backgroundColor,
          title: title != null
              ? LayoutBuilder(builder: (context, constraints) {
                  return Align(
                    alignment: Alignment.centerLeft,
                    child: Padding(
                      padding: const EdgeInsets.only(left: 8),
                      child: SizedBox(
                        width: constraints.maxWidth - 40,
                        child: Text(
                          title!,
                          maxLines: 3,
                          softWrap: true,
                          overflow: TextOverflow.fade,
                          style: const TextStyle(
                            fontSize: 15,
                          ).merge(titleTextStyle),
                        ),
                      ),
                    ),
                  );
                })
              : null,
          leading: leading ??
              (!navigateBackButton
                  ? null
                  : Align(
                      alignment: Alignment.centerLeft,
                      child: ThemedIconButton(
                        onPressed: () => Navigator.pop(context),
                        icon: ThemedIcons.arrowBack(),
                      ),
                    )),
          actions: actions != null
              ? Align(
                  alignment: Alignment.centerRight,
                  child: Row(
                    crossAxisAlignment: CrossAxisAlignment.center,
                    mainAxisAlignment: MainAxisAlignment.end,
                    children: actions!,
                  ),
                )
              : null,
          automaticallyImplyLeading: false,
        ),
      ),
    );
  }

  @override
  Size get preferredSize => Size(double.infinity, appBarHeight);
}
