// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:fluent_ui/fluent_ui.dart' as fluent;
import 'package:provider/provider.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class ThemedButton extends StatelessWidget {
  final Color? color;
  final Function() onPressed;
  final double borderRadiusPx;
  final bool offMode;
  final Widget? child;
  final Widget? Function(BuildContext context, ThemedColors currentTheme)?
      builder;
  final EdgeInsets? padding;
  final EdgeInsets? margin;

  const ThemedButton({
    Key? key,
    this.color,
    required this.onPressed,
    this.child,
    this.builder,
    this.borderRadiusPx = 25.0,
    this.offMode = false,
    this.padding = const EdgeInsets.all(5),
    this.margin,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Consumer<VertticeUIState>(builder: (context, value, _) {
      return FittedBox(
        fit: BoxFit.scaleDown,
        child: fluent.FilledButton(
          onPressed: onPressed,
          style: fluent.ButtonStyle(
            backgroundColor: offMode
                ? fluent.ButtonState.all(fluent.Colors.grey[150])
                : fluent.ButtonState.all(color),
          ),
          child: Center(
            child: Container(
              padding: padding,
              margin: margin,
              child: builder != null
                  ? builder!(context, value.themedColors)
                  : child,
            ),
          ),
        ),
      );
    });
  }
}
