// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:fluent_ui/fluent_ui.dart' as fluent;
import 'package:provider/provider.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class ThemedCircleAvatar extends StatelessWidget {
  final Widget? child;
  final Color? backgroundColor;
  final ImageProvider? backgroundImage;
  final double? radius;
  final BoxFit fit;
  const ThemedCircleAvatar({
    Key? key,
    this.child,
    this.backgroundColor,
    this.backgroundImage,
    this.radius,
    this.fit = BoxFit.scaleDown,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Consumer<VertticeUIState>(builder: (
      BuildContext context,
      VertticeUIState value,
      Widget? _,
    ) {
      return FittedBox(
        fit: fit,
        child: ConstrainedBox(
          constraints: const BoxConstraints(minWidth: 20, minHeight: 20),
          child: Container(
            decoration: BoxDecoration(
              color: backgroundColor ??
                  (value.isDarkTheme ? Colors.grey[850] : Colors.white),
              image: backgroundImage != null
                  ? DecorationImage(image: backgroundImage!)
                  : null,
              borderRadius: BorderRadius.all(
                Radius.circular(radius ?? 50),
              ),
              border: Border.all(
                width: 2,
                color: value.themedColors.accentColor,
                style: BorderStyle.solid,
              ),
            ),
            child: fluent.Center(
              child: fluent.Padding(
                padding: const EdgeInsets.all(1.5),
                child: child,
              ),
            ),
          ),
        ),
      );
    });
  }
}
