// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:flutter_quill/flutter_quill.dart' hide Text;

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';
import 'web_video_player.dart';
import 'web_youtube_player.dart';

Widget defaultEmbedBuilderWeb(
  BuildContext context,
  QuillController controller,
  Embed node,
  bool readOnly,
  void Function(GlobalKey videoContainerKey)? onVideoInit,
) {
  switch (node.value.type) {
    case BlockEmbed.imageType:
      final imageUrl = node.value.data;
      return ConstrainedBox(
        constraints: const BoxConstraints(maxHeight: 350),
        child: InkWell(
          onTap: () async {
            Navigator.push(
              context,
              MaterialPageRoute(
                builder: (BuildContext context) {
                  return ImageViewerFullPage(
                    imageID: imageUrl,
                    url: imageUrl,
                  );
                },
              ),
            );
          },
          child: Image.network(
            imageUrl,
            fit: BoxFit.contain,
          ),
        ),
      );
    case BlockEmbed.videoType:
      final videoUrl = node.value.data;
      if (videoUrl.contains('youtube.com') || videoUrl.contains('youtu.be')) {
        return TransparentPointer(
          child: WebYoutubeVideoApp(
            videoUrl: videoUrl,
            context: context,
            readOnly: readOnly,
          ),
        );
      }

      return WebVideoApp(
        videoUrl: videoUrl,
        context: context,
        readOnly: readOnly,
      );
    default:
      throw UnimplementedError(
        'Embeddable type "${node.value.type}" is not supported by default '
        'embed builder of QuillEditor. You must pass your own builder function '
        'to embedBuilder property of QuillEditor or QuillField widgets.',
      );
  }
}
