// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:collection/collection.dart';
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class EditorFileMangerView extends StatefulWidget {
  final List<FileModel> files;
  final String path;
  final Future<void> Function({
    required List<FileModel> files,
  }) filesDataCallback;
  const EditorFileMangerView({
    Key? key,
    required this.files,
    required this.path,
    required this.filesDataCallback,
  }) : super(key: key);

  @override
  State<EditorFileMangerView> createState() => _EditorFileMangerViewState();
}

class _EditorFileMangerViewState extends State<EditorFileMangerView> {
  late List<FileModel> _files;
  @override
  void initState() {
    super.initState();
    _files = widget.files;
  }

  @override
  void didUpdateWidget(covariant EditorFileMangerView oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const DeepCollectionEquality().equals(_files, widget.files)) {
      _files = widget.files;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: const ThemedAppBar(
        navigateBackButton: true,
        title: 'Editor files manager',
      ),
      body: Padding(
        padding: _files.isEmpty
            ? const EdgeInsets.all(50.0)
            : const EdgeInsets.all(20.0),
        child: FileUploadAndView(
          viewFromStorage: false,
          path: widget.path,
          filesDataCallback: ({
            required List<FileModel> files,
          }) async {
            setState(() {
              _files = [
                ..._files,
                ...files,
              ];
            });

            await widget.filesDataCallback(files: files);
          },
          files: _files,
          multi: true,
          onDoneUpload: () {
            setState(() {});
          },
        ),
      ),
    );
  }
}
