// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:fluent_ui/fluent_ui.dart' as fluent;

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class ThemedIconTextButton extends StatelessWidget {
  final String text;
  final TextStyle? textStyle;

  final Function() onPressed;
  final Icon icon;
  final bool showLoading;

  const ThemedIconTextButton(
    this.icon,
    this.text, {
    Key? key,
    required this.onPressed,
    this.showLoading = false,
    this.textStyle,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return FittedBox(
      fit: BoxFit.scaleDown,
      child: ElevatedButton.icon(
        onPressed: onPressed,
        icon: Icon(
          icon.icon!,
          size: icon.size ?? 20,
          color: icon.color,
        ),
        label: Row(
          children: [
            fluent.Text(
              text,
              style: ThemedFonts.primary(
                textStyle: textStyle,
              ),
            ),
            if (showLoading)
              const Padding(
                padding: EdgeInsets.symmetric(horizontal: 15),
                child: ThemedProgress(),
              )
          ],
        ),
      ),
    );
  }
}

class ThemedTextIconButton extends StatelessWidget {
  final String text;
  final TextStyle? textStyle;
  final Function() onPressed;
  final Icon icon;
  final bool showLoading;
  const ThemedTextIconButton(
    this.text,
    this.icon, {
    Key? key,
    required this.onPressed,
    this.showLoading = false,
    this.textStyle,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return FittedBox(
      fit: BoxFit.scaleDown,
      child: Directionality(
        textDirection: TextDirection.rtl,
        child: ElevatedButton.icon(
          onPressed: onPressed,
          icon: Icon(
            icon.icon!,
            size: icon.size ?? 20,
            color: icon.color,
          ),
          label: Row(
            children: [
              fluent.Text(
                text,
                style: ThemedFonts.primary(
                  textStyle: textStyle,
                ),
              ),
              if (showLoading)
                const Padding(
                  padding: EdgeInsets.symmetric(horizontal: 15),
                  child: ThemedProgress(),
                )
            ],
          ),
        ),
      ),
    );
  }
}
