// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:fluent_ui/fluent_ui.dart' as fluent;

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class ThemedMenuContent<T> extends StatefulWidget {
  final List<T> menuItems;
  final Widget Function(BuildContext context, int menuItemIndex, T item)
      content;
  final Widget Function(T)? itemIconBuilder;
  final String Function(T) itemStringLabelBuilder;
  const ThemedMenuContent({
    Key? key,
    required this.menuItems,
    required this.itemStringLabelBuilder,
    required this.content,
    this.itemIconBuilder,
  }) : super(key: key);

  @override
  State<ThemedMenuContent> createState() => _ThemedMenuContentState<T>();
}

class _ThemedMenuContentState<T> extends State<ThemedMenuContent<T>> {
  int _index = 0;
  final _autoSuggestBox = TextEditingController();

  @override
  Widget build(BuildContext context) {
    final paneItems = widget.menuItems
        .map(
          (e) => fluent.PaneItem(
            icon: widget.itemIconBuilder != null
                ? widget.itemIconBuilder!(e)
                : ThemedIcons.circle(),
            title: Text(widget.itemStringLabelBuilder(e)),
          ),
        )
        .toList();

    return fluent.NavigationView(
      pane: fluent.NavigationPane(
        // autoSuggestBox: ,
        selected: _index,
        items: paneItems,
        onChanged: (i) => setState(() => _index = i),
        displayMode: ScreenSizeUtils.isMobile(context: context)
            ? fluent.PaneDisplayMode.minimal
            : fluent.PaneDisplayMode.auto,
      ),
      content: widget.menuItems.isEmpty
          ? const SizedBox.shrink()
          : widget.content(
              context,
              _index,
              widget.menuItems[_index],
            ),
    );
  }
}
