// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:flutter_markdown/flutter_markdown.dart';
import 'package:html2md/html2md.dart' as html2md;

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class ThemedRichTextViewer extends StatefulWidget {
  final String? markdown;
  final String? html;
  ThemedRichTextViewer({
    Key? key,
    this.html,
    this.markdown,
  }) : super(key: key) {
    assert(
      html != null || markdown != null,
      'Must provide html or markdown',
    );
  }

  @override
  State<ThemedRichTextViewer> createState() => _ThemedRichTextViewerState();
}

class _ThemedRichTextViewerState extends State<ThemedRichTextViewer> {
  String? _markdown;
  @override
  void initState() {
    super.initState();
    if (widget.markdown != null) {
      _markdown = widget.markdown;
    } else {
      _iniMarkdown();
    }
  }

  _iniMarkdown() {
    var markdown = html2md.convert(widget.html!);

    setState(() {
      _markdown = markdown;
    });
  }

  @override
  Widget build(BuildContext context) {
    return _markdown == null
        ? centeredLoader
        : Padding(
            padding: const EdgeInsets.all(8.0),
            child: MarkdownBody(
              data: _markdown!,
              imageBuilder: (Uri url, String? _, String? __) {
                return ImageViewer(
                  height: 275,
                  url: url.toString(),
                );
              },
              selectable: true,
              onTapLink: (String text, String? href, String title) async {
                if (await canLaunchUrl(Uri.parse(href ?? ""))) {
                  launchUrl(Uri.parse(href!));
                }
              },
            ),
          );
  }
}
