// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:collection/collection.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class ThemedSearchMultiList<T> extends StatefulWidget {
  final List<Widget>? actions;
  final List<T> items;
  final Widget Function(T)? itemLabelBuilder;
  final String Function(T)? itemStringLabelBuilder;
  final Future<void> Function(List<T>) selectedItemsCallback;
  final List<T> selectedItems;
  final bool Function(String search, T item)? searchItemFn;
  final double? rowHeight;

  const ThemedSearchMultiList({
    Key? key,
    required this.items,
    required this.selectedItems,
    required this.selectedItemsCallback,
    this.itemLabelBuilder,
    this.itemStringLabelBuilder,
    this.actions,
    this.searchItemFn,
    this.rowHeight,
  }) : super(key: key);

  @override
  State<ThemedSearchMultiList> createState() =>
      _ThemedSearchMultiListState<T>();
}

class _ThemedSearchMultiListState<T> extends State<ThemedSearchMultiList<T>> {
  late List<T> _selectedItems;

  late List<T> _items;
  String _searchText = '';

  @override
  void initState() {
    super.initState();
    _selectedItems = widget.selectedItems;

    _items = widget.items;
  }

  @override
  void didUpdateWidget(covariant ThemedSearchMultiList<T> oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const DeepCollectionEquality().equals(_items, widget.items) ||
        !const DeepCollectionEquality()
            .equals(_selectedItems, widget.selectedItems)) {
      _items = widget.items;
      _selectedItems = widget.selectedItems;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        SizedBox(
          height: 57,
          child: Row(
            children: [
              Expanded(
                child: Padding(
                  padding: const EdgeInsets.symmetric(
                    horizontal: 6.0,
                    vertical: 3,
                  ),
                  child: TextFormFactory(
                    showStandardBorder: false,
                    displayName: 'Search',
                    name: 'search',
                    value: _searchText,
                    onChanged: (name, search) {
                      setState(() {
                        _searchText = search;
                      });
                    },
                  ),
                ),
              ),
              if (widget.actions != null)
                ...widget.actions!
                    .map((e) => Padding(
                          padding: const EdgeInsets.symmetric(horizontal: 4),
                          child: e,
                        ))
                    .toList(),
              InkWell(
                onTap: () async {
                  await widget.selectedItemsCallback(_selectedItems);
                },
                child: ThemedCircleAvatar(
                  child: ThemedIcons.check(),
                ),
              ),
            ],
          ),
        ),
        Expanded(
          child: Builder(builder: (context) {
            final items = _searchText == ''
                ? _items
                : _items.where((item) {
                    return widget.searchItemFn != null
                        ? widget.searchItemFn!(_searchText, item)
                        : item
                            .toString()
                            .toLowerCase()
                            .contains(_searchText.toLowerCase());
                  }).toList();
            return ListView.builder(
              itemCount: items.length,
              itemBuilder: (context, index) {
                final item = items[index];

                return Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Container(
                    height: widget.rowHeight,
                    padding: const EdgeInsets.all(13),
                    child: Row(
                      children: [
                        Expanded(
                          child: Builder(builder: (context) {
                            if (widget.itemLabelBuilder != null) {
                              return Center(
                                child: widget.itemLabelBuilder!(item),
                              );
                            }
                            if (widget.itemStringLabelBuilder != null) {
                              return Center(
                                child: Text(
                                  widget.itemStringLabelBuilder!(item),
                                  style: const TextStyle(
                                    fontSize: 12,
                                  ),
                                ),
                              );
                            }
                            return Center(
                              child: Text(
                                item.toString(),
                                style: const TextStyle(
                                  fontSize: 12,
                                ),
                              ),
                            );
                          }),
                        ),
                        Checkbox(
                          value: _selectedItems.contains(item),
                          onChanged: (bool? check) {
                            setState(() {
                              if (check != null && check) {
                                _selectedItems.add(item);
                              } else {
                                _selectedItems.remove(item);
                              }
                            });
                          },
                        ),
                      ],
                    ),
                  ),
                );
              },
            );
          }),
        ),
      ],
    );
  }
}
