import 'package:flutter/material.dart';
import 'package:verttice_ui/src/widgets/themed_widgets/themed_divider.dart';

import 'themed_sizes.dart';

enum ThemedFixedSide { left, right }

class ThemedSideBySidePage extends StatelessWidget {
  final double fixedSideWidth;
  final ThemedFixedSide fixedSide;
  final Widget Function(BuildContext context, Size size) fixedSideBuilder;
  final Widget Function(BuildContext context, Size size) flexSideBuilder;
  const ThemedSideBySidePage({
    Key? key,
    required this.fixedSideBuilder,
    required this.flexSideBuilder,
    this.fixedSide = ThemedFixedSide.left,
    this.fixedSideWidth = ThemedSizes.commonDesktopSideSelectMenu,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (context, constraints) {
        final page = [
          SizedBox(
            width: fixedSideWidth,
            height: constraints.maxHeight - ThemedSizes.commonDivider,
            child: fixedSideBuilder(
              context,
              Size(
                fixedSideWidth,
                constraints.maxHeight - ThemedSizes.commonDivider,
              ),
            ),
          ),
          ThemedDivider(
            height: constraints.maxHeight - ThemedSizes.commonDivider,
            width: ThemedSizes.commonDivider,
          ),
          SizedBox(
            width: constraints.maxWidth -
                (fixedSideWidth + ThemedSizes.commonDivider),
            height: constraints.maxHeight - ThemedSizes.commonDivider,
            child: flexSideBuilder(
              context,
              Size(
                constraints.maxWidth -
                    (fixedSideWidth + ThemedSizes.commonDivider),
                constraints.maxHeight - ThemedSizes.commonDivider,
              ),
            ),
          ),
        ];
        return Scaffold(
          body: SizedBox(
            width: constraints.maxWidth,
            height: constraints.maxHeight,
            child: Column(
              children: [
                ThemedDivider(
                  height: ThemedSizes.commonDivider,
                  width: constraints.maxWidth,
                ),
                Row(
                  children: fixedSide == ThemedFixedSide.left
                      ? page
                      : page.reversed.toList(),
                ),
              ],
            ),
          ),
        );
      },
    );
  }
}
