// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:collection/collection.dart';
import 'package:fluent_ui/fluent_ui.dart' as fluent;
import 'package:provider/provider.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class ThemedTabBarContent extends StatefulWidget {
  final List<ThemedMenuItem> items;
  final String? title;
  final TextStyle? titleTextStyle;
  final List<Widget>? actions;
  final bool navigateBackButton;
  final bool keepAlive;

  const ThemedTabBarContent({
    Key? key,
    required this.items,
    required this.navigateBackButton,
    this.title,
    this.actions,
    this.titleTextStyle,
    this.keepAlive = false,
  }) : super(key: key);

  @override
  State<ThemedTabBarContent> createState() => _ThemedTabBarContentState();
}

class _ThemedTabBarContentState extends State<ThemedTabBarContent> {
  int _index = 0;
  final viewKey = GlobalKey();

  @override
  Widget build(BuildContext context) {
    return Consumer<VertticeUIState>(builder: (context, value, _) {
      return LayoutBuilder(builder: (context, constrainst) {
        return SizedBox(
          height: constrainst.maxHeight,
          width: constrainst.maxWidth,
          child: Scaffold(
            key: viewKey,
            appBar: ThemedAppBar(
              navigateBackButton: widget.navigateBackButton,
              title: widget.title,
              titleTextStyle: widget.titleTextStyle,
              actions: widget.actions,
            ),
            body: Column(
              children: [
                SizedBox(
                  height: 50,
                  child: fluent.CommandBar(
                    isCompact: false,
                    primaryItems: widget.items
                        .mapIndexed(
                          (index, element) => fluent.CommandBarButton(
                            onPressed: (() => setState(() {
                                  _index = index;
                                })),
                            icon: Icon(
                              element.icon,
                              color: _index == index
                                  ? value.themedColors.accentColor
                                  : value.isDarkTheme
                                      ? Colors.white
                                      : Colors.grey[850],
                            ),
                            label: Text(
                              element.title,
                              style: TextStyle(
                                color: _index == index
                                    ? value.themedColors.accentColor
                                    : value.isDarkTheme
                                        ? Colors.white
                                        : Colors.grey[850],
                              ),
                            ),
                          ),
                        )
                        .toList(),
                  ),
                ),
                Expanded(
                    child: AnimatedContainer(
                  duration: const Duration(milliseconds: 400),
                  child: widget.items.isEmpty
                      ? const SizedBox.shrink()
                      : widget.keepAlive
                          ? fluent.IndexedStack(
                              index: _index,
                              children:
                                  widget.items.map((e) => e.child).toList(),
                            )
                          : widget.items.map((e) => e.child).toList()[_index],
                ))
              ],
            ),
          ),
        );
      });
    });
  }
}

class ThemedMenuItem {
  final String title;
  final IconData icon;
  final Widget child;

  ThemedMenuItem({
    required this.title,
    required this.icon,
    required this.child,
  });
}
