// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:provider/provider.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class ThemedText extends StatelessWidget {
  final String text;
  final TextStyle? style;
  final Alignment alignment;
  final bool selectable;
  final bool showThemed;
  final BoxFit fit;
  final bool useScreenConstraints;

  const ThemedText(
    this.text, {
    Key? key,
    this.style,
    this.alignment = Alignment.center,
    this.selectable = false,
    this.showThemed = false,
    this.fit = BoxFit.scaleDown,
    this.useScreenConstraints = true,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Consumer<VertticeUIState>(builder: (context, value, _) {
      final newStyle = ThemedFonts.primary(
        background: style?.background,
        backgroundColor: style?.backgroundColor,
        decoration: style?.decoration,
        decorationColor: style?.decorationColor,
        decorationStyle: style?.decorationStyle,
        decorationThickness: style?.decorationThickness,
        fontFeatures: style?.fontFeatures,
        fontSize: style?.fontSize ?? (showThemed ? 20 : null),
        fontStyle: style?.fontStyle,
        fontWeight: style?.fontWeight ?? FontWeight.normal,
        letterSpacing: style?.letterSpacing,
        color: style?.color ??
            (showThemed ? value.themedColors.accentColor : null),
        shadows: style?.shadows,
        textBaseline: style?.textBaseline,
        wordSpacing: style?.wordSpacing,
      );
      if (!useScreenConstraints) {
        if (selectable) {
          return FittedBox(
            fit: fit,
            child: Align(
              alignment: alignment,
              child: SelectableText(
                text,
                style: newStyle,
                minLines: 1,
              ),
            ),
          );
        }
        return FittedBox(
          fit: fit,
          child: Align(
            alignment: alignment,
            child: Text(
              text,
              style: newStyle,
              overflow: style?.overflow ?? TextOverflow.fade,
              softWrap: true,
            ),
          ),
        );
      }

      return LayoutBuilder(builder: (context, constraints) {
        if (selectable) {
          return FittedBox(
            fit: fit,
            child: SizedBox(
              width: constraints.maxWidth,
              child: Align(
                alignment: alignment,
                child: SelectableText(
                  text,
                  style: newStyle,
                  minLines: 1,
                ),
              ),
            ),
          );
        }
        return FittedBox(
          fit: fit,
          child: SizedBox(
            width: constraints.maxWidth,
            child: Align(
              alignment: alignment,
              child: Text(
                text,
                style: newStyle,
                overflow: style?.overflow ?? TextOverflow.fade,
                softWrap: true,
              ),
            ),
          ),
        );
      });
    });
  }
}
