// 🎯 Dart imports:
import 'dart:ui';

// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:sensors_plus/sensors_plus.dart';

class ThreeDImageEffect extends StatefulWidget {
  final String url;
  const ThreeDImageEffect({
    Key? key,
    required this.url,
  }) : super(key: key);

  @override
  State<ThreeDImageEffect> createState() => _ThreeDImageEffectState();
}

class _ThreeDImageEffectState extends State<ThreeDImageEffect> {
  double initX = 0.0, initY = 0.0;

  @override
  Widget build(BuildContext context) {
    return ConstrainedBox(
      constraints: const BoxConstraints(maxHeight: 315, maxWidth: 400),
      child: Stack(
        children: [
          StreamBuilder<GyroscopeEvent>(
              stream: SensorsPlatform.instance.gyroscopeEvents,
              builder: (context, snapshot) {
                if (snapshot.hasData) {
                  if (snapshot.data!.y.abs() > 0.0) {
                    initX = initX + (snapshot.data!.y);
                  }
                  if (snapshot.data!.x.abs() > 0.0) {
                    initY = initY + (snapshot.data!.x);
                  }
                }
                return Positioned(
                  left: 10 - initX,
                  right: 10 + initX,
                  top: 10 - initY,
                  bottom: 10 + initY,
                  width: 310,
                  height: 210,
                  child: Center(
                    child: Stack(
                      children: [
                        Padding(
                          padding: const EdgeInsets.all(15.0),
                          child: Container(
                            width: 310,
                            height: 210,
                            decoration: BoxDecoration(
                              image: DecorationImage(
                                  isAntiAlias: true,
                                  opacity: 0.8,
                                  image: NetworkImage(widget.url),
                                  colorFilter: ColorFilter.mode(
                                      Colors.white.withOpacity(.1),
                                      BlendMode.srcOver),
                                  fit: BoxFit.cover),
                              borderRadius: BorderRadius.circular(10),
                            ),
                            child: BackdropFilter(
                              filter: ImageFilter.blur(sigmaX: 4, sigmaY: 4),
                              child: Container(
                                decoration: BoxDecoration(
                                    borderRadius: BorderRadius.circular(10),
                                    color: Colors.white.withOpacity(0.0)),
                              ),
                            ),
                          ),
                        ),
                      ],
                    ),
                  ),
                );
              }),
          Positioned(
            left: 10,
            right: 10,
            top: 10,
            bottom: 10,
            width: 350,
            height: 250,
            child: Center(
              child: Container(
                width: 350,
                height: 250,
                decoration: BoxDecoration(
                  border: Border.all(color: Colors.white, width: .1),
                  image: DecorationImage(
                      image: NetworkImage(widget.url), fit: BoxFit.cover),
                  borderRadius: BorderRadius.circular(10),
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }
}
