library verttice_ui;

// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:flutter_requery/flutter_requery.dart' as qc;

// 🌎 Project imports:
import 'package:verttice_ui/src/utils/screen_size.dart';

// themed app
export 'src/app.dart';

// packages
export 'package:after_layout/after_layout.dart';
export 'package:url_launcher/url_launcher.dart';
export 'package:flutter_hooks/flutter_hooks.dart';

// config
export 'src/config/config.dart';

// const
export 'src/const/box_sizes.dart';
export 'src/const/icons.dart';
export 'src/const/loaders.dart';
export 'src/const/paddings.dart';
export 'src/const/spacers.dart';

// state
export 'src/state/verttice_ui_state.dart';

export 'src/tools/screen_type.dart';

// enums
export 'src/enums/screen_type_enum.dart';

// utils
export 'src/utils/color_by_number.dart';
export 'src/utils/download.dart';
export 'src/utils/screen_size.dart';
export 'src/utils/html_tags_cleaner.dart';
export 'src/utils/platform.dart';
export 'src/utils/excel_to_map.dart';

// widgets
// dialogs
export 'src/widgets/dialogs/dialogs_index.dart';

// drawer
export 'src/widgets/drawing/painter.dart';

// widget/files
export 'src/widgets/files/files_index.dart';

// maps
export 'src/widgets/maps/maps_index.dart';

// widget/data_factory
export 'src/widgets/data_factory/data_factory_index.dart';

// widgets/themed_widgets
export 'src/widgets/themed_widgets/themed_widgets_index.dart';

// widgets/routes
export 'src/widgets/routes/navigator_page.dart';

export 'src/widgets/widgets_index.dart';

// classes export
class VertticeImages {
  static const String logoAndSymbol =
      "https://files.verttice.co.uk/p/api/v1/files/verttice/dashboard/k8ZpJfynyuBVZM291eT0K427ciWu5fXOk9irdPv63aFMMQG7MdOC73fpAaJ";
  static const String symbol =
      "https://files.verttice.co.uk/p/api/v1/files/verttice/dashboard/AzAZ20rCpTobczvNrXDtr3UgMDaSiQ7gGk4TuTBSqzzJeVkwczRjpc";
  static const String logo =
      "https://files.verttice.co.uk/p/api/v1/files/verttice/dashboard/XwClTvPvU8HXjEPXALvlNEEOQ3ly3nOteUN8Y7EjrHO4h2ii";

  static const String backgroundDesktop =
      "https://files.verttice.co.uk/p/api/v1/files/verttice/dashboard/Hzv1DiiCgFYlZ3aQFmqYNgeftEkWzVMu9VL1UcsOtrFGLR8OYPQ";
  static const String backgroundMobile =
      "https://files.verttice.co.uk/p/api/v1/files/verttice/dashboard/vcR2pAc6jstC7kkMjOWUCMqkUemzoemqXKXeWSXAfiAtdQTXIkWJu7XAGi";
  static const String backgroundDesktopPng =
      "https://files.verttice.co.uk/p/api/v1/files/verttice/dashboard/nZtCuWmMmiASuy4th7iFnGvDcyPwabb1KT1CX4L1zUa3rHOcqqvjKllHp";
  static const String backgroundMobilePng =
      "https://files.verttice.co.uk/p/api/v1/files/verttice/dashboard/m8kSCqJaaEaxiEmZ9xYExYzV7Eok8aoD9bcx45kgjqvKzMhj50pOvl";
  String vertticePlatformBg(BuildContext context) {
    return ScreenSizeUtils.isMobile(context: context)
        ? backgroundMobilePng
        : backgroundDesktopPng;
  }

  static const vertticeBgCredit =
      'https://unsplash.com/photos/xfngap_DToE?utm_source=unsplash&utm_medium=referral&utm_content=creditShareLink';
}

// packages
class RequeryResponse<T> implements qc.QueryResponse<T> {
  @override
  T? data;

  @override
  bool loading;

  @override
  dynamic error;

  RequeryResponse({
    this.data,
    this.loading = false,
    this.error,
  });
}

class Requery {
  static final queryCache = qc.queryCache;

  static qc.Query<T> query<T>({
    required String key,
    required Future<T> Function() future,
    required Widget Function({
      required BuildContext context,
      required RequeryResponse<T> response,
    })
        builder,
  }) {
    return qc.Query<T>(
      key,
      future: future,
      builder: (BuildContext context, qc.QueryResponse<T> response) {
        return builder(
          context: context,
          response: RequeryResponse<T>(
            data: response.data,
            error: response.error,
            loading: response.loading,
          ),
        );
      },
    );
  }
}
